@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
+    copyright (c) 2009--2011 johannes hanika.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ void dt_opencl_init(dt_opencl_t *cl, const int argc, char *argv[])
 
   // look for explicit definition of opencl_runtime library in preferences
   const char *library = dt_conf_get_string("opencl_runtime");
-  dt_print(DT_DEBUG_OPENCL, "[opencl_init] trying to load opencl library: %s\n", library && strlen(library) != 0 ? library : "<system default>");
+  dt_print(DT_DEBUG_OPENCL, "[opencl_init] trying to load opencl library: '%s'\n", library && strlen(library) != 0 ? library : "<system default>");
 
   // dynamically load opencl runtime
   if(!dt_dlopencl_init(library, &cl->dlocl))
@@ -60,7 +60,7 @@ void dt_opencl_init(dt_opencl_t *cl, const int argc, char *argv[])
     }
     else
     {
-      dt_print(DT_DEBUG_OPENCL, "[opencl_init] opencl library %s found on your system and loaded\n", cl->dlocl->library);
+      dt_print(DT_DEBUG_OPENCL, "[opencl_init] opencl library '%s' found on your system and loaded\n", cl->dlocl->library);
     }
 
   cl_int err;
@@ -100,37 +100,40 @@ void dt_opencl_init(dt_opencl_t *cl, const int argc, char *argv[])
   {
     memset(cl->dev[dev].program_used, 0x0, sizeof(int)*DT_OPENCL_MAX_PROGRAMS);
     memset(cl->dev[dev].kernel_used,  0x0, sizeof(int)*DT_OPENCL_MAX_KERNELS);
+    cl->dev[dev].eventlist = NULL;
+    cl->dev[dev].eventnames = NULL;
+    cl->dev[dev].numevents = 0;
+    cl->dev[dev].maxevents = 0;
+    cl->dev[dev].used_global_mem = 0;
     cl_device_id devid = cl->dev[dev].devid = devices[k];
 
     char infostr[1024];
     size_t infoint;
     size_t infointtab[1024];
     cl_bool image_support = 0;
-    cl_ulong max_global_mem = 0;
-    //size_t image_width = 0, image_height = 0;
 
     // test 1GB mem and image support:
     (cl->dlocl->symbols->dt_clGetDeviceInfo)(devid, CL_DEVICE_NAME, sizeof(infostr), &infostr, NULL);
     (cl->dlocl->symbols->dt_clGetDeviceInfo)(devid, CL_DEVICE_IMAGE_SUPPORT, sizeof(cl_bool), &image_support, NULL);
     (cl->dlocl->symbols->dt_clGetDeviceInfo)(devid, CL_DEVICE_IMAGE2D_MAX_HEIGHT, sizeof(size_t), &(cl->dev[dev].max_image_height), NULL);
     (cl->dlocl->symbols->dt_clGetDeviceInfo)(devid, CL_DEVICE_IMAGE2D_MAX_WIDTH,  sizeof(size_t), &(cl->dev[dev].max_image_width),  NULL);
+    (cl->dlocl->symbols->dt_clGetDeviceInfo)(devid, CL_DEVICE_MAX_MEM_ALLOC_SIZE,  sizeof(cl_ulong), &(cl->dev[dev].max_mem_alloc),  NULL);
     if(!image_support)
     {
       dt_print(DT_DEBUG_OPENCL, "[opencl_init] discarding device %d `%s' due to missing image support.\n", k, infostr);
       continue;
     }
-    //if(image_height < 8192 || image_width < 8192)
-    //{
-    //  fprintf(stderr, "[opencl_init] WARNING: your card only supports image sizes of %zd x %zd\n", image_width, image_height);
-    //  fprintf(stderr, "[opencl_init] WARNING: expect random crashes, especially with images larger than that.\n");
-    //}
 
-    (cl->dlocl->symbols->dt_clGetDeviceInfo)(devid, CL_DEVICE_GLOBAL_MEM_SIZE, sizeof(cl_ulong), &max_global_mem, NULL);
-    if(max_global_mem < 1000000000ul)
+    (cl->dlocl->symbols->dt_clGetDeviceInfo)(devid, CL_DEVICE_GLOBAL_MEM_SIZE, sizeof(cl_ulong), &(cl->dev[dev].max_global_mem), NULL);
+    if(cl->dev[dev].max_global_mem < 1000000000ul)
     {
-      dt_print(DT_DEBUG_OPENCL, "[opencl_init] discarding device %d `%s' due to insufficient global memory (%luMB).\n", k, infostr, max_global_mem/1024/1024);
+      dt_print(DT_DEBUG_OPENCL, "[opencl_init] discarding device %d `%s' due to insufficient global memory (%luMB).\n", k, infostr, cl->dev[dev].max_global_mem/1024/1024);
       continue;
     }
+
+    dt_print(DT_DEBUG_OPENCL, "[opencl_init] device %d `%s' supports image sizes of %zd x %zd\n", k, infostr, cl->dev[dev].max_image_width, cl->dev[dev].max_image_height);
+    dt_print(DT_DEBUG_OPENCL, "[opencl_init] device %d `%s' allows GPU memory allocations of up to %luMB\n", k, infostr, cl->dev[dev].max_mem_alloc/1024/1024);
+
     if(darktable.unmuted & DT_DEBUG_OPENCL)
     {
       printf("[opencl_init] device %d: %s \n", k, infostr);
@@ -152,7 +155,7 @@ void dt_opencl_init(dt_opencl_t *cl, const int argc, char *argv[])
       return;
     }
     // create a command queue for first device the context reported
-    cl->dev[dev].cmd_queue = (cl->dlocl->symbols->dt_clCreateCommandQueue)(cl->dev[dev].context, devid, 0, &err);
+    cl->dev[dev].cmd_queue = (cl->dlocl->symbols->dt_clCreateCommandQueue)(cl->dev[dev].context, devid, (darktable.unmuted & DT_DEBUG_PERF) ? CL_QUEUE_PROFILING_ENABLE : 0, &err);
     if(err != CL_SUCCESS)
     {
       dt_print(DT_DEBUG_OPENCL, "[opencl_init] could not create command queue for device %d: %d\n", k, err);
@@ -180,8 +183,8 @@ void dt_opencl_init(dt_opencl_t *cl, const int argc, char *argv[])
         if(programname[0] == '\0') continue;
         snprintf(filename, 1024, "%s/kernels/%s", dtpath, programname);
         dt_print(DT_DEBUG_OPENCL, "[opencl_init] compiling program `%s' ..\n", programname);
-        const int prog = dt_opencl_load_program(cl, k, filename);
-        if(dt_opencl_build_program(cl, k, prog))
+        const int prog = dt_opencl_load_program(k, filename);
+        if(dt_opencl_build_program(k, prog))
         {
           dt_print(DT_DEBUG_OPENCL, "[opencl_init] failed to compile program `%s'!\n", programname);
           return;
@@ -223,6 +226,9 @@ void dt_opencl_cleanup(dt_opencl_t *cl)
       for(int k=0; k<DT_OPENCL_MAX_PROGRAMS; k++) if(cl->dev[i].program_used[k]) (cl->dlocl->symbols->dt_clReleaseProgram)(cl->dev[i].program[k]);
       (cl->dlocl->symbols->dt_clReleaseCommandQueue)(cl->dev[i].cmd_queue);
       (cl->dlocl->symbols->dt_clReleaseContext)(cl->dev[i].context);
+      dt_opencl_events_reset(i);
+      free(cl->dev[i].eventlist);
+      free(cl->dev[i].eventnames);
     }
 
   if(cl->dlocl) {
@@ -233,14 +239,23 @@ void dt_opencl_cleanup(dt_opencl_t *cl)
   dt_pthread_mutex_destroy(&cl->lock);
 }
 
-void dt_opencl_finish(cl_command_queue q)
+int dt_opencl_finish(const int devid)
 {
-  if(!darktable.opencl->inited) return;
-  (darktable.opencl->dlocl->symbols->dt_clFinish)(q);
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || devid < 0) return -1;
+  return (cl->dlocl->symbols->dt_clFinish)(cl->dev[devid].cmd_queue);
 }
 
-int dt_opencl_lock_device(dt_opencl_t *cl, const int _dev)
+int dt_opencl_enqueue_barrier(const int devid)
 {
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || devid < 0) return -1;
+  return (cl->dlocl->symbols->dt_clEnqueueBarrier)(cl->dev[devid].cmd_queue);
+}
+
+int dt_opencl_lock_device(const int _dev)
+{
+  dt_opencl_t *cl = darktable.opencl;
   if(!cl->inited) return -1;
   int dev = _dev;
   if(dev < 0 || dev >= cl->num_devs) dev = 0;
@@ -250,21 +265,24 @@ int dt_opencl_lock_device(dt_opencl_t *cl, const int _dev)
     const int try_dev = (dev + i) % cl->num_devs;
     if(!dt_pthread_mutex_trylock(&cl->dev[try_dev].lock)) return try_dev;
   }
-  // no free GPU :( lock the requested one, after blocking.
-  dt_pthread_mutex_lock(&cl->dev[dev].lock);
-  return dev;
+  // no free GPU :(
+  // use CPU processing, if no free device:
+  if(!dt_pthread_mutex_trylock(&cl->dev[dev].lock)) return dev;
+  return -1;
 }
 
-void dt_opencl_unlock_device(dt_opencl_t *cl, const int dev)
+void dt_opencl_unlock_device(const int dev)
 {
+  dt_opencl_t *cl = darktable.opencl;
   if(!cl->inited) return;
   if(dev < 0 || dev >= cl->num_devs) return;
   dt_pthread_mutex_unlock(&cl->dev[dev].lock);
 }
 
-int dt_opencl_load_program(dt_opencl_t *cl, const int dev, const char *filename)
+int dt_opencl_load_program(const int dev, const char *filename)
 {
   cl_int err;
+  dt_opencl_t *cl = darktable.opencl;
   FILE *f = fopen(filename, "rb");
   if(!f)
   {
@@ -327,9 +345,10 @@ int dt_opencl_load_program(dt_opencl_t *cl, const int dev, const char *filename)
   }
 }
 
-int dt_opencl_build_program(dt_opencl_t *cl, const int dev, const int prog)
+int dt_opencl_build_program(const int dev, const int prog)
 {
   if(prog < 0 || prog >= DT_OPENCL_MAX_PROGRAMS) return -1;
+  dt_opencl_t *cl = darktable.opencl;
   cl_program program = cl->dev[dev].program[prog];
   cl_int err;
   err = (cl->dlocl->symbols->dt_clBuildProgram)(program, 1, &cl->dev[dev].devid, "-cl-fast-relaxed-math -cl-strict-aliasing", 0, 0);
@@ -361,8 +380,9 @@ int dt_opencl_build_program(dt_opencl_t *cl, const int dev, const int prog)
   return err;
 }
 
-int dt_opencl_create_kernel(dt_opencl_t *cl, const int prog, const char *name)
+int dt_opencl_create_kernel(const int prog, const char *name)
 {
+  dt_opencl_t *cl = darktable.opencl;
   if(!cl->inited) return -1;
   if(prog < 0 || prog >= DT_OPENCL_MAX_PROGRAMS) return -1;
   dt_pthread_mutex_lock(&cl->lock);
@@ -399,8 +419,9 @@ error:
   return -1;
 }
 
-void dt_opencl_free_kernel(dt_opencl_t *cl, const int kernel)
+void dt_opencl_free_kernel(const int kernel)
 {
+  dt_opencl_t *cl = darktable.opencl;
   if(!cl->inited) return;
   if(kernel < 0 || kernel >= DT_OPENCL_MAX_KERNELS) return;
   dt_pthread_mutex_lock(&cl->lock);
@@ -412,50 +433,81 @@ void dt_opencl_free_kernel(dt_opencl_t *cl, const int kernel)
   dt_pthread_mutex_unlock(&cl->lock);
 }
 
-int dt_opencl_get_max_work_item_sizes(dt_opencl_t *cl, const int dev, size_t *sizes)
+int dt_opencl_get_max_work_item_sizes(const int dev, size_t *sizes)
 {
-  if(!cl->inited) return -1;
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || dev < 0) return -1;
   return (cl->dlocl->symbols->dt_clGetDeviceInfo)(cl->dev[dev].devid, CL_DEVICE_MAX_WORK_ITEM_SIZES, sizeof(size_t)*3, sizes, NULL);
 }
 
-int dt_opencl_set_kernel_arg(dt_opencl_t *cl, const int dev, const int kernel, const int num, const size_t size, const void *arg)
+int dt_opencl_set_kernel_arg(const int dev, const int kernel, const int num, const size_t size, const void *arg)
 {
-  if(!cl->inited) return -1;
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || dev < 0) return -1;
   if(kernel < 0 || kernel >= DT_OPENCL_MAX_KERNELS) return -1;
   return (cl->dlocl->symbols->dt_clSetKernelArg)(cl->dev[dev].kernel[kernel], num, size, arg);
 }
 
-int dt_opencl_enqueue_kernel_2d(dt_opencl_t *cl, const int dev, const int kernel, const size_t *sizes)
+int dt_opencl_enqueue_kernel_2d(const int dev, const int kernel, const size_t *sizes)
 {
-  if(!cl->inited) return -1;
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || dev < 0) return -1;
   if(kernel < 0 || kernel >= DT_OPENCL_MAX_KERNELS) return -1;
+  int err;
+  char buf[256];
+  buf[0]='\0';
+  (cl->dlocl->symbols->dt_clGetKernelInfo)(cl->dev[dev].kernel[kernel], CL_KERNEL_FUNCTION_NAME, 256, buf, NULL);
+  cl_event *eventp = dt_opencl_events_get_slot(dev, buf);
   // const size_t local[2] = {16, 16};
   // let the driver choose:
   const size_t *local = NULL;
-  return (cl->dlocl->symbols->dt_clEnqueueNDRangeKernel)(cl->dev[dev].cmd_queue, cl->dev[dev].kernel[kernel], 2, NULL, sizes, local, 0, NULL, NULL);
+  err = (cl->dlocl->symbols->dt_clEnqueueNDRangeKernel)(cl->dev[dev].cmd_queue, cl->dev[dev].kernel[kernel], 2, NULL, sizes, local, 0, NULL, eventp);
+  // if (err == CL_SUCCESS) err = dt_opencl_finish(dev);
+  return err;
 }
 
-int dt_opencl_copy_device_to_host(void *host, void *device, const int width, const int height, const int devid, const int bpp)
+
+int dt_opencl_copy_device_to_host(const int devid, void *host, void *device, const int width, const int height, const int bpp)
 {
-  if(!darktable.opencl->inited) return -1;
+  return dt_opencl_read_host_from_device(devid, host, device, width, height, bpp);
+}
+
+
+int dt_opencl_read_host_from_device(const int devid, void *host, void *device, const int width, const int height, const int bpp)
+{
+  if(!darktable.opencl->inited || devid < 0) return -1;
   size_t origin[] = {0, 0, 0};
   size_t region[] = {width, height, 1};
+  cl_event *eventp = dt_opencl_events_get_slot(devid, "[Read Image (from device to host)]");
   // blocking.
-  return (darktable.opencl->dlocl->symbols->dt_clEnqueueReadImage)(darktable.opencl->dev[devid].cmd_queue, device, CL_TRUE, origin, region, region[0]*bpp, 0, host, 0, NULL, NULL);
+  return (darktable.opencl->dlocl->symbols->dt_clEnqueueReadImage)(darktable.opencl->dev[devid].cmd_queue, device, CL_TRUE, origin, region, region[0]*bpp, 0, host, 0, NULL, eventp);
 }
 
-int dt_opencl_enqueue_copy_image(cl_command_queue q, cl_mem src, cl_mem dst, size_t *orig_src, size_t *orig_dst, size_t *region, int events, cl_event *wait, cl_event *event)
+
+int dt_opencl_write_host_to_device(const int devid, void *host, void *device, const int width, const int height, const int bpp)
 {
-  if(!darktable.opencl->inited) return -1;
+  if(!darktable.opencl->inited || devid < 0) return -1;
+  size_t origin[] = {0, 0, 0};
+  size_t region[] = {width, height, 1};
+  cl_event *eventp = dt_opencl_events_get_slot(devid, "[Write Image (from host to device)]");
+  // blocking.
+  return (darktable.opencl->dlocl->symbols->dt_clEnqueueWriteImage)(darktable.opencl->dev[devid].cmd_queue, device, CL_TRUE, origin, region, region[0]*bpp, 0, host, 0, NULL, eventp);
+}
+
+
+int dt_opencl_enqueue_copy_image(const int devid, cl_mem src, cl_mem dst, size_t *orig_src, size_t *orig_dst, size_t *region)
+{
+  if(!darktable.opencl->inited || devid < 0) return -1;
   cl_int err;
-  err = (darktable.opencl->dlocl->symbols->dt_clEnqueueCopyImage)(q, src, dst, orig_src, orig_dst, region, events, wait, event);
+  cl_event *eventp = dt_opencl_events_get_slot(devid, "[Copy Image (on device)]");
+  err = (darktable.opencl->dlocl->symbols->dt_clEnqueueCopyImage)(darktable.opencl->dev[devid].cmd_queue, src, dst, orig_src, orig_dst, region, 0, NULL, eventp);
   if(err != CL_SUCCESS) dt_print(DT_DEBUG_OPENCL, "[opencl copy_image] could not copy image: %d\n", err);
   return err;
 }
 
-void* dt_opencl_copy_host_to_device_constant(const int size, const int devid, void *host)
+void* dt_opencl_copy_host_to_device_constant(const int devid, const int size, void *host)
 {
-  if(!darktable.opencl->inited) return NULL;
+  if(!darktable.opencl->inited || devid < 0) return NULL;
   cl_int err;
   cl_mem dev = (darktable.opencl->dlocl->symbols->dt_clCreateBuffer) (darktable.opencl->dev[devid].context,
                                CL_MEM_READ_ONLY|CL_MEM_COPY_HOST_PTR,
@@ -465,9 +517,9 @@ void* dt_opencl_copy_host_to_device_constant(const int size, const int devid, vo
   return dev;
 }
 
-void* dt_opencl_copy_host_to_device(void *host, const int width, const int height, const int devid, const int bpp)
+void* dt_opencl_copy_host_to_device(const int devid, void *host, const int width, const int height, const int bpp)
 {
-  if(!darktable.opencl->inited) return NULL;
+  if(!darktable.opencl->inited || devid < 0) return NULL;
   cl_int err;
   cl_image_format fmt;
   // guess pixel format from bytes per pixel
@@ -494,7 +546,7 @@ void* dt_opencl_copy_host_to_device(void *host, const int width, const int heigh
                                 &fmt,
                                 width, height, 0,
                                 host, &err);
-  if(err != CL_SUCCESS) dt_print(DT_DEBUG_OPENCL, "[opencl copy_host_to_device] could not alloc/copy img buffer onto device %d: %d\n", devid, err);
+  if(err != CL_SUCCESS) dt_print(DT_DEBUG_OPENCL, "[opencl copy_host_to_device] could not alloc/copy img buffer on device %d: %d\n", devid, err);
   return dev;
 }
 
@@ -506,9 +558,9 @@ void dt_opencl_release_mem_object(void *mem)
 }
 
 
-void* dt_opencl_alloc_device(const int width, const int height, const int devid, const int bpp)
+void* dt_opencl_alloc_device(const int devid, const int width, const int height, const int bpp)
 {
-  if(!darktable.opencl->inited) return NULL;
+  if(!darktable.opencl->inited || devid < 0) return NULL;
   cl_int err;
   cl_image_format fmt;
   // guess pixel format from bytes per pixel
@@ -539,10 +591,43 @@ void* dt_opencl_alloc_device(const int width, const int height, const int devid,
 }
 
 
+void* dt_opencl_alloc_device_use_host_pointer(const int devid, const int width, const int height, const int bpp, void *host)
+{
+  if(!darktable.opencl->inited || devid < 0) return NULL;
+  cl_int err;
+  cl_image_format fmt;
+  // guess pixel format from bytes per pixel
+  if(bpp == 4*sizeof(float))
+    fmt = (cl_image_format)
+  {
+    CL_RGBA, CL_FLOAT
+  };
+  else if(bpp == sizeof(float))
+    fmt = (cl_image_format)
+  {
+    CL_R, CL_FLOAT
+  };
+  else if(bpp == sizeof(uint16_t))
+    fmt = (cl_image_format)
+  {
+    CL_R, CL_UNSIGNED_INT16
+  };
+  else return NULL;
+
+  cl_mem dev = (darktable.opencl->dlocl->symbols->dt_clCreateImage2D) (darktable.opencl->dev[devid].context,
+                                CL_MEM_READ_WRITE | ((host == NULL) ? CL_MEM_ALLOC_HOST_PTR : CL_MEM_USE_HOST_PTR),
+                                &fmt,
+                                width, height, 0,
+                                host, &err);
+  if(err != CL_SUCCESS) dt_print(DT_DEBUG_OPENCL, "[opencl alloc_device_use_host_pointer] could not alloc img buffer on device %d: %d\n", devid, err);
+  return dev;
+}
+
+
 /** check if image size fit into limits given by OpenCL runtime */
 int dt_opencl_image_fits_device(const int devid, const size_t width, const size_t height)
 {
-  if(!darktable.opencl->inited) return FALSE;
+  if(!darktable.opencl->inited || devid < 0) return FALSE;
 
   return (darktable.opencl->dev[devid].max_image_width >= width && darktable.opencl->dev[devid].max_image_height >= height);
 }
@@ -573,9 +658,9 @@ void dt_opencl_disable(void)
 
 
 /** update enabled flag with value from preferences */
-void dt_opencl_update_enabled(void)
+int dt_opencl_update_enabled(void)
 {
-  if(!darktable.opencl->inited) return;
+  if(!darktable.opencl->inited) return FALSE;
   const int prefs = dt_conf_get_bool("opencl");
 
   //printf("[opencl_update_enabled] preferences is set to %d\n", prefs);
@@ -585,6 +670,281 @@ void dt_opencl_update_enabled(void)
     darktable.opencl->enabled = prefs;
     dt_print(DT_DEBUG_OPENCL, "[opencl_update_enabled] enabled flag set to %s\n", prefs ? "ON" : "OFF");    
   }
+  return darktable.opencl->enabled;
+}
+
+/** get global memory of device */
+cl_ulong dt_opencl_get_max_global_mem(const int devid)
+{
+  if(!darktable.opencl->inited || devid < 0) return 0;
+  return darktable.opencl->dev[devid].max_global_mem;
+}
+
+
+/** the following eventlist functions assume that affected structures are locked upstream */
+
+/** get next free slot in eventlist (and manage size of eventlist) */
+cl_event *dt_opencl_events_get_slot(const int devid, const char *tag)
+{
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || devid < 0) return NULL;
+
+  static const cl_event zeroevent[1];   // implicitly initialized to zero
+  cl_event **eventlist = &(cl->dev[devid].eventlist);
+  char **eventnames = &(cl->dev[devid].eventnames);
+  int *numevents = &(cl->dev[devid].numevents);
+  int *maxevents = &(cl->dev[devid].maxevents);
+
+  // if first time called: allocate initial buffers
+  if (*eventlist == NULL)
+  {
+    int newevents = DT_OPENCL_EVENTLISTSIZE;
+    *eventlist = malloc(newevents*sizeof(cl_event));
+    *eventnames = malloc(newevents*DT_OPENCL_EVENTNAMELENGTH);
+    if (!*eventlist || !*eventnames)
+    {
+      free(*eventlist);
+      free(*eventnames);
+      *eventlist=NULL;
+      *eventnames=NULL;
+      return NULL;
+    }
+    *maxevents = newevents;
+  }
+
+  // check if currently highest event slot was actually consumed. If not use it again
+  if (*numevents > 0 && !memcmp(*eventlist+*numevents-1, zeroevent, sizeof(cl_event)))
+  {
+    if (tag != NULL)
+    {
+      strncpy(*eventnames+(*numevents-1)*DT_OPENCL_EVENTNAMELENGTH, tag, DT_OPENCL_EVENTNAMELENGTH);
+    }
+    else
+    {
+      *(*eventnames+(*numevents-1)*DT_OPENCL_EVENTNAMELENGTH)='\0';
+    }
+    return *eventlist+*numevents-1;
+  }
+
+  // if no more space left in eventlist: grow buffer		
+  if (*numevents == *maxevents)
+  {
+    int newevents = 2 * (*maxevents);
+    cl_event *neweventlist = malloc(newevents*sizeof(cl_event));
+    char *neweventnames = malloc(newevents*DT_OPENCL_EVENTNAMELENGTH);
+    if (!neweventlist || !neweventnames)
+    {
+      free(neweventlist);
+      free(neweventnames);
+      return NULL;
+    }
+    memcpy(neweventlist, *eventlist, *maxevents*sizeof(cl_event));
+    memcpy(neweventnames, *eventnames, *maxevents*DT_OPENCL_EVENTNAMELENGTH);	
+    free(*eventlist);
+    free(*eventnames);
+    *eventlist = neweventlist;
+    *eventnames = neweventnames;
+    *maxevents = newevents;
+  }
+
+  // init next event slot and return it
+  (*numevents)++;
+  memcpy(*eventlist + *numevents-1, zeroevent, sizeof(cl_event));
+  if (tag != NULL)
+  {
+    strncpy(*eventnames+(*numevents-1)*DT_OPENCL_EVENTNAMELENGTH, tag, DT_OPENCL_EVENTNAMELENGTH);
+  }
+  else
+  {
+    *(*eventnames+(*numevents-1)*DT_OPENCL_EVENTNAMELENGTH)='\0';
+  }
+  return *eventlist+*numevents-1;
+}
+
+
+/** reset eventlist to empty state */
+void dt_opencl_events_reset(const int devid)
+{
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || devid < 0) return;
+
+  cl_event **eventlist = &(cl->dev[devid].eventlist);
+  int *numevents = &(cl->dev[devid].numevents);
+
+  if (*eventlist == NULL || *numevents == 0) return; // nothing to do
+
+  // release all events in eventlist, not to waste resources
+  for (int k = 0; k < *numevents; k++)
+  {
+    (cl->dlocl->symbols->dt_clReleaseEvent)((*eventlist)[k]);
+  }
+  *numevents=0;
+  return;
+}
+
+
+/** Wait for events in eventlist to terminate -> this is a blocking synchronization point!
+    Does not flush eventlist */
+void dt_opencl_events_wait_for(const int devid)
+{
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || devid < 0) return;
+
+  static const cl_event zeroevent[1];   // implicitly initialized to zero
+  cl_event **eventlist = &(cl->dev[devid].eventlist);
+  int *numevents = &(cl->dev[devid].numevents);
+
+  if (*eventlist==NULL || *numevents==0) return; // nothing to do
+
+  // check if last event slot was acutally used and correct eventcount if needed	
+  int eventcount = *numevents;
+  if (!memcmp(*eventlist+eventcount-1, zeroevent, sizeof(cl_event))) eventcount--;
+
+  // now wait for all events to terminate
+  // Risk: might never return in case of OpenCL blocks or endless loops
+  // TODO: run clWaitForEvents in separate thread and implement watchdog timer
+  (cl->dlocl->symbols->dt_clWaitForEvents)(eventcount, *eventlist);
+
+  return;
+}
+
+
+/** Wait for events in eventlist to terminate, check for return status of events and
+    report summary success info (CL_COMPLETE or last error code) */
+cl_int dt_opencl_events_flush(const int devid, const int retain)
+{
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || devid < 0) return FALSE;
+
+  static const cl_event zeroevent[1];   // implicitly initialized to zero
+  cl_event **eventlist = &(cl->dev[devid].eventlist);
+  char **eventnames = &(cl->dev[devid].eventnames);
+  int *numevents = &(cl->dev[devid].numevents);
+
+  if (*eventlist==NULL || *numevents==0) return CL_COMPLETE; // nothing to do, no news is good news
+
+  // Wait for command queue to terminate
+  dt_opencl_events_wait_for(devid);
+
+  // check if last event slot was acutally used and correct eventcount if needed	
+  int eventcount = *numevents;
+  if (!memcmp(*eventlist+eventcount-1, zeroevent, sizeof(cl_event))) eventcount--;
+
+  // now check return status of all terminated events	
+  cl_int err;
+  cl_int retval;
+  cl_int summary=CL_COMPLETE;
+  for (int k = 0; k < eventcount; k++)
+  {
+    char *tag = *eventnames+k*DT_OPENCL_EVENTNAMELENGTH;
+    err = (cl->dlocl->symbols->dt_clGetEventInfo)((*eventlist)[k], CL_EVENT_COMMAND_EXECUTION_STATUS, sizeof(cl_int), &retval, NULL);
+    if (err != CL_SUCCESS)
+    {
+      dt_print(DT_DEBUG_OPENCL, "[opencl_events_flush] could not get event info for '%s': %d\n", tag[0] == '\0' ? "<?>" : tag, err);
+      continue;
+    }
+
+    if (retval != CL_COMPLETE)
+    {
+      dt_print(DT_DEBUG_OPENCL, "[opencl_events_flush] execution of '%s' %s: %d\n", tag[0] == '\0' ? "<?>" : tag, retval == CL_COMPLETE ? "was successful" : "failed", retval);
+      summary=retval;
+    }
+  }
+
+  // give profiling info if wanted
+  if (darktable.unmuted & DT_DEBUG_PERF)
+    dt_opencl_events_profiling(devid, 1);
+
+  // if retain is not set: reset eventlist
+  if (!retain) dt_opencl_events_reset(devid);
+
+  return summary;
+}
+
+
+/** display OpenCL profiling information. If summary is not 0, try to generate summarized info for kernels */
+void dt_opencl_events_profiling(const int devid, int summary)
+{
+  dt_opencl_t *cl = darktable.opencl;
+  if(!cl->inited || devid < 0) return;
+
+  cl_event **eventlist = &(cl->dev[devid].eventlist);
+  char **eventnames = &(cl->dev[devid].eventnames);
+  int *numevents = &(cl->dev[devid].numevents);
+
+  if (*eventlist == NULL || *numevents == 0) return; // nothing to do
+
+  cl_int errs;
+  cl_int erre;
+  cl_ulong start;
+  cl_ulong end;
+  float timelapsed;
+  char *tags[*numevents+1];
+  float timings[*numevents+1];
+  int items = 1;
+  tags[0] = "";
+  timings[0] = 0.0f;
+
+  // get profiling info and arrange it
+  for (int k = 0; k < *numevents; k++)
+  {
+    errs = (cl->dlocl->symbols->dt_clGetEventProfilingInfo)((*eventlist)[k], CL_PROFILING_COMMAND_START, sizeof(cl_ulong), &start, NULL);
+    erre = (cl->dlocl->symbols->dt_clGetEventProfilingInfo)((*eventlist)[k], CL_PROFILING_COMMAND_END, sizeof(cl_ulong), &end, NULL);
+
+    if (errs != CL_SUCCESS || erre != CL_SUCCESS) continue;
+
+    // translate from nanoseconds to seconds
+    timelapsed = (end - start) / 1.0e9;
+
+    // if summary is not FALSE, try to sum up timings of kernels
+    if (summary)
+    {
+      // this is not efficient at all
+      int tagfound = -1;
+      for (int i=0; i<items; i++)
+      {
+        if (!strcmp(tags[i], *eventnames+k*DT_OPENCL_EVENTNAMELENGTH))
+        {
+          tagfound=i;
+          break;
+        }
+      }
+
+      if (tagfound >= 0) // tag was already detected before
+      {
+        timings[tagfound] += timelapsed;
+      }
+      else // tag is new
+      {
+	items++;
+        tags[items-1] = *eventnames+k*DT_OPENCL_EVENTNAMELENGTH;
+        timings[items-1] = timelapsed;
+      }
+    }
+
+    else // no summary info wanted -> arrange event by event
+    {
+      items++;
+      tags[items-1] = *eventnames+k*DT_OPENCL_EVENTNAMELENGTH;
+      timings[items-1] = timelapsed;
+    }
+  }
+
+  // now display profiling info
+  float total = 0.0f;
+  for (int i=1; i<items; i++)
+  {
+    dt_print(DT_DEBUG_OPENCL, "[opencl_profiling] spent %7.4f seconds in %s\n", (double)timings[i], tags[i][0] == '\0' ? "<?>" : tags[i]);
+    total += timings[i];
+  }
+  // timing info for items without tag (if any)
+  if (timings[0] != 0.0f)
+  {		
+    dt_print(DT_DEBUG_OPENCL, "[opencl_profiling] spent %7.4f seconds (unallocated)\n", (double)timings[0]);
+    total += timings[0];
+  }
+  dt_print(DT_DEBUG_OPENCL, "[opencl_profiling] spent %7.4f seconds totally in command queue\n", (double)total);
+
   return;
 }
 
