@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
+    copyright (c) 2009--2011 johannes hanika.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "control/conf.h"
 #include "common/image_cache.h"
 #include "common/imageio.h"
+#include "common/tags.h"
 #include "common/debug.h"
 #include "gui/gtk.h"
 
@@ -66,6 +67,8 @@ void dt_dev_set_gamma_array(dt_develop_t *dev, const float linear, const float g
 
 void dt_dev_init(dt_develop_t *dev, int32_t gui_attached)
 {
+  dev->closures = NULL;
+
   float downsampling = dt_conf_get_float ("preview_subsample");
   dev->preview_downsampling = downsampling <= 1.0 && downsampling >= 0.1 ? downsampling : .5;
   dev->gui_module = NULL;
@@ -597,7 +600,6 @@ void dt_dev_add_history_item(dt_develop_t *dev, dt_iop_module_t *module, gboolea
 {
   if(darktable.gui->reset) return;
   dt_pthread_mutex_lock(&dev->history_mutex);
-  if (dev && dev->image) dev->image->dirty = 1;
   if(dev->gui_attached)
   {
     // if gui_attached pop all operations down to dev->history_end
@@ -731,17 +733,10 @@ void dt_dev_reload_history_items(dt_develop_t *dev)
   dt_dev_pop_history_items(dev, dev->history_end);
 }
 
-/**
- * \brief Pop items off the history stack until only at most (cnt) remain
- *
- * \param[in] dev the developer to use for state
- * \param[in] cnt the maximum number of elements to leave on the history stack
- */
 void dt_dev_pop_history_items(dt_develop_t *dev, int32_t cnt)
 {
   // printf("dev popping all history items >= %d\n", cnt);
   dt_pthread_mutex_lock(&dev->history_mutex);
-  if (dev && dev->image) dev->image->dirty = 1;
   darktable.gui->reset = 1;
   dev->history_end = cnt;
   // reset gui params for all modules
@@ -781,14 +776,10 @@ void dt_dev_pop_history_items(dt_develop_t *dev, int32_t cnt)
   dt_control_queue_draw_all();
 }
 
-/**
- * \brief Write updated history to DB for the active image in the developer
- *
- * \param[in] dev The developer to use for state
- */
 void dt_dev_write_history(dt_develop_t *dev)
 {
   sqlite3_stmt *stmt;
+  gboolean changed = FALSE;
   DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "delete from history where imgid = ?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, dev->image->id);
   sqlite3_step(stmt);
@@ -799,7 +790,17 @@ void dt_dev_write_history(dt_develop_t *dev)
     dt_dev_history_item_t *hist = (dt_dev_history_item_t *)(history->data);
     (void)dt_dev_write_history_item(dev->image, hist, i);
     history = g_list_next(history);
+    changed = TRUE;
   }
+  
+  /* attach / detach changed tag reflecting actual change */
+  guint tagid = 0;
+  dt_tag_new("darktable|changed",&tagid); 
+  if(changed)
+    dt_tag_attach(tagid, dev->image->id);
+  else
+    dt_tag_detach(tagid, dev->image->id);
+
 }
 
 void dt_dev_read_history(dt_develop_t *dev)
