@@ -27,6 +27,7 @@
 #include "gui/presets.h"
 #include "dtgtk/button.h"
 #include "dtgtk/slider.h"
+// #include "dtgtk/tristatebutton.h" //FIXME
 
 #include <strings.h>
 #include <assert.h>
@@ -231,11 +232,11 @@ int dt_iop_load_module_so(dt_iop_module_so_t *module, const char *libname, const
   if(!g_module_symbol(module->module, "gui_cleanup",            (gpointer)&(module->gui_cleanup)))            goto error;
 
   if(!g_module_symbol(module->module, "gui_post_expose",        (gpointer)&(module->gui_post_expose)))        module->gui_post_expose = NULL;
+  if(!g_module_symbol(module->module, "init_key_accels", (gpointer)&(module->init_key_accels)))        module->init_key_accels = NULL;
   if(!g_module_symbol(module->module, "mouse_leave",            (gpointer)&(module->mouse_leave)))            module->mouse_leave = NULL;
   if(!g_module_symbol(module->module, "mouse_moved",            (gpointer)&(module->mouse_moved)))            module->mouse_moved = NULL;
   if(!g_module_symbol(module->module, "button_released",        (gpointer)&(module->button_released)))        module->button_released = NULL;
   if(!g_module_symbol(module->module, "button_pressed",         (gpointer)&(module->button_pressed)))         module->button_pressed = NULL;
-  if(!g_module_symbol(module->module, "key_pressed",            (gpointer)&(module->key_pressed)))            module->key_pressed = NULL;
   if(!g_module_symbol(module->module, "configure",              (gpointer)&(module->configure)))              module->configure = NULL;
   if(!g_module_symbol(module->module, "scrolled",               (gpointer)&(module->scrolled)))               module->scrolled = NULL;
 
@@ -305,7 +306,6 @@ dt_iop_load_module_by_so(dt_iop_module_t *module, dt_iop_module_so_t *so, dt_dev
   module->mouse_moved     = so->mouse_moved;
   module->button_released = so->button_released;
   module->button_pressed  = so->button_pressed;
-  module->key_pressed     = so->key_pressed;
   module->configure       = so->configure;
   module->scrolled        = so->scrolled;
 
@@ -439,7 +439,7 @@ void dt_iop_load_modules_so()
   GList *res = NULL;
   dt_iop_module_so_t *module;
   darktable.iop = NULL;
-  char plugindir[1024], op[20];
+  char plugindir[1024], op[20], accelpath[1024];
   const gchar *d_name;
   dt_get_plugindir(plugindir, 1024);
   g_strlcat(plugindir, "/plugins", 1024);
@@ -463,6 +463,23 @@ void dt_iop_load_modules_so()
     g_free(libname);
     res = g_list_append(res, module);
     init_presets(module);
+    // Calling the accelerator initialization callback, if present
+    if(module->init_key_accels)
+      (module->init_key_accels)();
+
+    if(!(module->flags() & IOP_FLAGS_DEPRECATED))
+    {
+      // Adding the optional show accelerator to the table (blank)
+      snprintf(accelpath, 256, "<Darktable>/darkroom/plugins/%s/show plugin",
+               (module->op));
+      gtk_accel_map_add_entry(accelpath, 0, 0);
+      dt_accel_group_connect_by_path(darktable.control->accels_darkroom, accelpath,
+                                     NULL);
+      snprintf(accelpath, 1024, "<Darktable>/darkroom/plugins/%s/reset plugin parameters",module->op);
+      dtgtk_button_init_accel(darktable.control->accels_darkroom,accelpath);
+      snprintf(accelpath, 1024, "<Darktable>/darkroom/plugins/%s/show preset menu",module->op);
+      dtgtk_button_init_accel(darktable.control->accels_darkroom,accelpath);
+    }
   }
   g_dir_close(dir);
   darktable.iop = res;
@@ -511,17 +528,18 @@ void dt_iop_cleanup_module(dt_iop_module_t *module)
 {
   free(module->factory_params); module->factory_params = NULL ;  
   module->cleanup(module);
+
   free(module->default_params); module->default_params = NULL ; 
   if (module->blend_params != NULL) 
-    {
-      free(module->blend_params) ; 
-      module->blend_params = NULL ; 
-    }
-   if (module->default_blendop_params != NULL) 
-    {
-      free(module->default_blendop_params) ; 
-      module->default_blendop_params = NULL ; 
-    }
+  {
+    free(module->blend_params) ; 
+    module->blend_params = NULL ; 
+  }
+  if (module->default_blendop_params != NULL) 
+  {
+    free(module->default_blendop_params) ; 
+    module->default_blendop_params = NULL ; 
+  }
   dt_pthread_mutex_destroy(&module->params_mutex);
 }
 
@@ -641,7 +659,7 @@ expander_button_callback(GtkWidget *widget, GdkEventButton *event, dt_iop_module
       /* if module is the current, always expand it */
       if(m==module)
         gtk_expander_set_expanded(m->expander, TRUE);
-      else if((current_group == 0 || (current_group & m->groups()) ))
+      else if((current_group == 0 || (current_group & m->groups()) || (current_group == IOP_SPECIAL_GROUP_ACTIVE_PIPE && m->enabled) /* || (current_group == IOP_SPECIAL_GROUP_USER_DEFINED && module->showhide && dtgtk_tristatebutton_get_state (DTGTK_TRISTATEBUTTON(module->showhide))==2) */ )) //FIXME
         gtk_expander_set_expanded(m->expander, FALSE);
 
       iop = g_list_next(iop);
@@ -826,7 +844,11 @@ GtkWidget *dt_iop_gui_get_expander(dt_iop_module_t *module)
 
   gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(module->expander), TRUE, TRUE, 0);
   GtkDarktableButton *resetbutton = DTGTK_BUTTON(dtgtk_button_new(dtgtk_cairo_paint_reset, CPF_STYLE_FLAT|CPF_DO_NOT_USE_BORDER));
+  snprintf(name, 1024, "<Darktable>/darkroom/plugins/%s/reset plugin parameters",module->op);
+  dtgtk_button_set_accel(resetbutton,darktable.control->accels_darkroom,name);
   GtkDarktableButton *presetsbutton = DTGTK_BUTTON(dtgtk_button_new(dtgtk_cairo_paint_presets, CPF_STYLE_FLAT|CPF_DO_NOT_USE_BORDER));
+  snprintf(name, 1024, "<Darktable>/darkroom/plugins/%s/show preset menu",module->op);
+  dtgtk_button_set_accel(presetsbutton,darktable.control->accels_darkroom,name);
   gtk_widget_set_size_request(GTK_WIDGET(presetsbutton),13,13);
   gtk_widget_set_size_request(GTK_WIDGET(resetbutton),13,13);
   g_object_set(G_OBJECT(resetbutton), "tooltip-text", _("reset parameters"), (char *)NULL);
@@ -1267,4 +1289,82 @@ void dt_iop_YCbCr_to_RGB(const float *yuv, float *rgb)
   rgb[2] = yuv[0] + 2.028*yuv[1];
 }
 
+static inline void
+mat4inv(const float X[][4], float R[][4])
+{
+  const float det = 
+          X[0][3] * X[1][2] * X[2][1] * X[3][0]
+        - X[0][2] * X[1][3] * X[2][1] * X[3][0] 
+        - X[0][3] * X[1][1] * X[2][2] * X[3][0] 
+        + X[0][1] * X[1][3] * X[2][2] * X[3][0] 
+        + X[0][2] * X[1][1] * X[2][3] * X[3][0] 
+        - X[0][1] * X[1][2] * X[2][3] * X[3][0] 
+        - X[0][3] * X[1][2] * X[2][0] * X[3][1] 
+        + X[0][2] * X[1][3] * X[2][0] * X[3][1] 
+        + X[0][3] * X[1][0] * X[2][2] * X[3][1] 
+        - X[0][0] * X[1][3] * X[2][2] * X[3][1] 
+        - X[0][2] * X[1][0] * X[2][3] * X[3][1] 
+        + X[0][0] * X[1][2] * X[2][3] * X[3][1] 
+        + X[0][3] * X[1][1] * X[2][0] * X[3][2] 
+        - X[0][1] * X[1][3] * X[2][0] * X[3][2] 
+        - X[0][3] * X[1][0] * X[2][1] * X[3][2] 
+        + X[0][0] * X[1][3] * X[2][1] * X[3][2] 
+        + X[0][1] * X[1][0] * X[2][3] * X[3][2] 
+        - X[0][0] * X[1][1] * X[2][3] * X[3][2] 
+        - X[0][2] * X[1][1] * X[2][0] * X[3][3] 
+        + X[0][1] * X[1][2] * X[2][0] * X[3][3] 
+        + X[0][2] * X[1][0] * X[2][1] * X[3][3] 
+        - X[0][0] * X[1][2] * X[2][1] * X[3][3] 
+        - X[0][1] * X[1][0] * X[2][2] * X[3][3] 
+        + X[0][0] * X[1][1] * X[2][2] * X[3][3];
+  R[0][0] = ( X[1][2]*X[2][3]*X[3][1] - X[1][3]*X[2][2]*X[3][1] + X[1][3]*X[2][1]*X[3][2] - X[1][1]*X[2][3]*X[3][2] - X[1][2]*X[2][1]*X[3][3] + X[1][1]*X[2][2]*X[3][3] ) / det;
+  R[1][0] = ( X[1][3]*X[2][2]*X[3][0] - X[1][2]*X[2][3]*X[3][0] - X[1][3]*X[2][0]*X[3][2] + X[1][0]*X[2][3]*X[3][2] + X[1][2]*X[2][0]*X[3][3] - X[1][0]*X[2][2]*X[3][3] ) / det;
+  R[2][0] = ( X[1][1]*X[2][3]*X[3][0] - X[1][3]*X[2][1]*X[3][0] + X[1][3]*X[2][0]*X[3][1] - X[1][0]*X[2][3]*X[3][1] - X[1][1]*X[2][0]*X[3][3] + X[1][0]*X[2][1]*X[3][3] ) / det;
+  R[3][0] = ( X[1][2]*X[2][1]*X[3][0] - X[1][1]*X[2][2]*X[3][0] - X[1][2]*X[2][0]*X[3][1] + X[1][0]*X[2][2]*X[3][1] + X[1][1]*X[2][0]*X[3][2] - X[1][0]*X[2][1]*X[3][2] ) / det;
+
+  R[0][1] = ( X[0][3]*X[2][2]*X[3][1] - X[0][2]*X[2][3]*X[3][1] - X[0][3]*X[2][1]*X[3][2] + X[0][1]*X[2][3]*X[3][2] + X[0][2]*X[2][1]*X[3][3] - X[0][1]*X[2][2]*X[3][3] ) / det;
+  R[1][1] = ( X[0][2]*X[2][3]*X[3][0] - X[0][3]*X[2][2]*X[3][0] + X[0][3]*X[2][0]*X[3][2] - X[0][0]*X[2][3]*X[3][2] - X[0][2]*X[2][0]*X[3][3] + X[0][0]*X[2][2]*X[3][3] ) / det;
+  R[2][1] = ( X[0][3]*X[2][1]*X[3][0] - X[0][1]*X[2][3]*X[3][0] - X[0][3]*X[2][0]*X[3][1] + X[0][0]*X[2][3]*X[3][1] + X[0][1]*X[2][0]*X[3][3] - X[0][0]*X[2][1]*X[3][3] ) / det;
+  R[3][1] = ( X[0][1]*X[2][2]*X[3][0] - X[0][2]*X[2][1]*X[3][0] + X[0][2]*X[2][0]*X[3][1] - X[0][0]*X[2][2]*X[3][1] - X[0][1]*X[2][0]*X[3][2] + X[0][0]*X[2][1]*X[3][2] ) / det;
+
+  R[0][2] = ( X[0][2]*X[1][3]*X[3][1] - X[0][3]*X[1][2]*X[3][1] + X[0][3]*X[1][1]*X[3][2] - X[0][1]*X[1][3]*X[3][2] - X[0][2]*X[1][1]*X[3][3] + X[0][1]*X[1][2]*X[3][3] ) / det;
+  R[1][2] = ( X[0][3]*X[1][2]*X[3][0] - X[0][2]*X[1][3]*X[3][0] - X[0][3]*X[1][0]*X[3][2] + X[0][0]*X[1][3]*X[3][2] + X[0][2]*X[1][0]*X[3][3] - X[0][0]*X[1][2]*X[3][3] ) / det;
+  R[2][2] = ( X[0][1]*X[1][3]*X[3][0] - X[0][3]*X[1][1]*X[3][0] + X[0][3]*X[1][0]*X[3][1] - X[0][0]*X[1][3]*X[3][1] - X[0][1]*X[1][0]*X[3][3] + X[0][0]*X[1][1]*X[3][3] ) / det;
+  R[3][2] = ( X[0][2]*X[1][1]*X[3][0] - X[0][1]*X[1][2]*X[3][0] - X[0][2]*X[1][0]*X[3][1] + X[0][0]*X[1][2]*X[3][1] + X[0][1]*X[1][0]*X[3][2] - X[0][0]*X[1][1]*X[3][2] ) / det;
+
+  R[0][3] = ( X[0][3]*X[1][2]*X[2][1] - X[0][2]*X[1][3]*X[2][1] - X[0][3]*X[1][1]*X[2][2] + X[0][1]*X[1][3]*X[2][2] + X[0][2]*X[1][1]*X[2][3] - X[0][1]*X[1][2]*X[2][3] ) / det;
+  R[1][3] = ( X[0][2]*X[1][3]*X[2][0] - X[0][3]*X[1][2]*X[2][0] + X[0][3]*X[1][0]*X[2][2] - X[0][0]*X[1][3]*X[2][2] - X[0][2]*X[1][0]*X[2][3] + X[0][0]*X[1][2]*X[2][3] ) / det;
+  R[2][3] = ( X[0][3]*X[1][1]*X[2][0] - X[0][1]*X[1][3]*X[2][0] - X[0][3]*X[1][0]*X[2][1] + X[0][0]*X[1][3]*X[2][1] + X[0][1]*X[1][0]*X[2][3] - X[0][0]*X[1][1]*X[2][3] ) / det;
+  R[3][3] = ( X[0][1]*X[1][2]*X[2][0] - X[0][2]*X[1][1]*X[2][0] + X[0][2]*X[1][0]*X[2][1] - X[0][0]*X[1][2]*X[2][1] - X[0][1]*X[1][0]*X[2][2] + X[0][0]*X[1][1]*X[2][2] ) / det;
+}
+
+static void
+mat4mulv (float *dst, float mat[][4], const float *const v)
+{
+  for(int k=0; k<4; k++)
+  {
+    float x=0.0f;
+    for(int i=0; i<4; i++) x += mat[k][i] * v[i];
+    dst[k] = x;
+  }
+}
+
+void dt_iop_estimate_cubic(const float *const x, const float *const y, float *a)
+{
+  // we want to fit a spline
+  // [y]   [x^3 x^2 x^1 1] [a^3]
+  // |y| = |x^3 x^2 x^1 1| |a^2|
+  // |y|   |x^3 x^2 x^1 1| |a^1|
+  // [y]   [x^3 x^2 x^1 1] [ 1 ]
+  // and do that by inverting the matrix X:
+
+  const float X[4][4] = {{x[0]*x[0]*x[0], x[0]*x[0], x[0], 1.0f},
+                         {x[1]*x[1]*x[1], x[1]*x[1], x[1], 1.0f},
+                         {x[2]*x[2]*x[2], x[2]*x[2], x[2], 1.0f},
+                         {x[3]*x[3]*x[3], x[3]*x[3], x[3], 1.0f}};
+  float X_inv[4][4];
+  mat4inv(X, X_inv);
+  mat4mulv(a, X_inv, y);
+}
+
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
