@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
+    copyright (c) 2009--2011 johannes hanika.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -482,7 +482,7 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
          pipe->opencl_error, return with value 1, and leave appropriate action to the calling function */
 
       /* try to run opencl module after checking some pre-requisites */
-      if(dt_opencl_is_enabled() && module->process_cl && piece->process_cl_ready  && 
+      if(pipe->opencl_enabled && module->process_cl && piece->process_cl_ready  && 
          dt_opencl_image_fits_device(pipe->devid, roi_in.width, roi_in.height) && 
          dt_opencl_image_fits_device(pipe->devid, roi_out->width, roi_out->height))
       {
@@ -497,7 +497,7 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
         }
         else
         {
-          cl_mem_input = dt_opencl_copy_host_to_device(input, roi_in.width, roi_in.height, pipe->devid, in_bpp);
+          cl_mem_input = dt_opencl_copy_host_to_device(pipe->devid, input, roi_in.width, roi_in.height, in_bpp);
           if (cl_mem_input == NULL)
           {
             dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] couldn't generate input buffer for module %s\n", module->op);
@@ -508,7 +508,7 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
         /* try to allocate GPU memory for output */
         if (success_opencl)
         {
-          *cl_mem_output = dt_opencl_alloc_device(roi_out->width, roi_out->height, pipe->devid, bpp);
+          *cl_mem_output = dt_opencl_alloc_device(pipe->devid, roi_out->width, roi_out->height, bpp);
           if (*cl_mem_output == NULL)
           {
             dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] couldn't allocate output buffer for module %s\n", module->op);
@@ -560,7 +560,7 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
             cl_int err;
 
             /* copy back to CPU buffer, then clean unneeded buffer */
-            err = dt_opencl_copy_device_to_host(input, cl_mem_input, roi_in.width, roi_in.height, pipe->devid, in_bpp);
+            err = dt_opencl_copy_device_to_host(pipe->devid, input, cl_mem_input, roi_in.width, roi_in.height, in_bpp);
             if (err != CL_SUCCESS)
             {
               /* fatal opencl error */
@@ -592,7 +592,7 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
         {
           cl_int err;
 
-          err = dt_opencl_copy_device_to_host(input, cl_mem_input, roi_in.width, roi_in.height, pipe->devid, in_bpp);
+          err = dt_opencl_copy_device_to_host(pipe->devid, input, cl_mem_input, roi_in.width, roi_in.height, in_bpp);
           // if (rand() % 5 == 0) err = !CL_SUCCESS; // Test code: simulate spurious failures
           if (err != CL_SUCCESS)
           {
@@ -776,7 +776,7 @@ dt_dev_pixelpipe_process_rec_and_backcopy(dt_dev_pixelpipe_t *pipe, dt_develop_t
     {
       cl_int err;
 
-      err = dt_opencl_copy_device_to_host(*output, *cl_mem_output, roi_out->width, roi_out->height, pipe->devid, *out_bpp);
+      err = dt_opencl_copy_device_to_host(pipe->devid, *output, *cl_mem_output, roi_out->width, roi_out->height, *out_bpp);
       dt_opencl_release_mem_object(*cl_mem_output);
       *cl_mem_output = NULL;
 
@@ -801,9 +801,10 @@ dt_dev_pixelpipe_process_rec_and_backcopy(dt_dev_pixelpipe_t *pipe, dt_develop_t
 int dt_dev_pixelpipe_process(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, int x, int y, int width, int height, float scale)
 {
   pipe->processing = 1;
-  dt_opencl_update_enabled(); // update enabled flag from preferences
-  pipe->devid = dt_opencl_lock_device(darktable.opencl, -1);
+  pipe->opencl_enabled = dt_opencl_update_enabled(); // update enabled flag from preferences
+  pipe->devid = dt_opencl_lock_device(-1);
   dt_print(DT_DEBUG_OPENCL, "[pixelpipe_process] [%s] using device %d\n", pipe->type == DT_DEV_PIXELPIPE_PREVIEW ? "preview" : (pipe->type == DT_DEV_PIXELPIPE_FULL ? "full" : "export"), pipe->devid);
+  dt_opencl_events_reset(pipe->devid);
   dt_iop_roi_t roi = (dt_iop_roi_t)
   {
     x, y, width, height, scale
@@ -817,33 +818,46 @@ int dt_dev_pixelpipe_process(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, int x,
   GList *modules = g_list_last(dev->iop);
   GList *pieces = g_list_last(pipe->nodes);
 
-  // in case of fatal opencl errors (not recoverable ones) we disable opencl and
-  // start all over again
+  // re-entry point: in case of fatal OpenCL errors we start all over again with OpenCL disabled
 restart:
   // image max is normalized before
   for(int k=0; k<3; k++) pipe->processed_maximum[k] = 1.0f; // dev->image->maximum;
   void *buf = NULL;
   void *cl_mem_out = NULL;
   int out_bpp;
-  if (dt_dev_pixelpipe_process_rec_and_backcopy(pipe, dev, &buf, &cl_mem_out, &out_bpp, &roi, modules, pieces, pos))
+  // run pixelpipe recursively and get error status
+  int err = dt_dev_pixelpipe_process_rec_and_backcopy(pipe, dev, &buf, &cl_mem_out, &out_bpp, &roi, modules, pieces, pos);
+  // check error status of OpenCL queue
+  int oclerr = (dt_opencl_events_flush(pipe->devid, 0) != CL_COMPLETE);
+
+  // OpenCL errors can come in two ways: pipe->opencl_error is TRUE or oclerr is TRUE
+  // if we have OpenCL errors ....
+  if (oclerr || (err && pipe->opencl_error))
+  {
+    // we might need to free an invalid OpenCL memory object
+    if (cl_mem_out != NULL) dt_opencl_release_mem_object(cl_mem_out);
+    // dt_opencl_disable();
+    // dt_control_log("Warning: OpenCL was found to be unreliable on this system and is therefore disabled!");
+    dt_pthread_mutex_lock(&pipe->busy_mutex);
+    pipe->opencl_enabled = 0; // disable OpenCL for this pipe
+    pipe->opencl_error = 0;
+    dt_dev_pixelpipe_flush_caches(pipe);
+    dt_dev_pixelpipe_change(pipe, dev);
+    dt_pthread_mutex_unlock(&pipe->busy_mutex);
+    goto restart;  // (as said before)
+  }
+
+  // release resources:
+  dt_opencl_unlock_device(pipe->devid);
+  pipe->devid = -1;
+  // ... and in case of other errors ...
+  if (err)
   {
-    if (pipe->opencl_error)
-    {
-      if (cl_mem_out != NULL) dt_opencl_release_mem_object(cl_mem_out);
-      dt_opencl_disable();
-      dt_control_log("Warning: OpenCL was found to be unreliable on this system and is therefore disabled!");
-      dt_pthread_mutex_lock(&pipe->busy_mutex);
-      pipe->opencl_error = 0;
-      dt_dev_pixelpipe_flush_caches(pipe);
-      dt_dev_pixelpipe_change(pipe, dev);
-      dt_pthread_mutex_unlock(&pipe->busy_mutex);
-      goto restart;
-    }
     pipe->processing = 0;
-    dt_opencl_unlock_device(darktable.opencl, pipe->devid);
-    pipe->devid = -1;
     return 1;
   }
+
+  // terminate
   dt_pthread_mutex_lock(&pipe->backbuf_mutex);
   pipe->backbuf_hash = dt_dev_pixelpipe_cache_hash(dev->image->id, &roi, pipe, 0);
   pipe->backbuf = buf;
@@ -853,8 +867,6 @@ restart:
 
   // printf("pixelpipe homebrew process end\n");
   pipe->processing = 0;
-  dt_opencl_unlock_device(darktable.opencl, pipe->devid);
-  pipe->devid = -1;
   return 0;
 }
 
