@@ -1,6 +1,7 @@
 /*
 		This file is part of darktable,
-		copyright (c) 2009--2010 johannes hanika.
+		copyright (c) 2009--2011 johannes hanika.
+		copyright (c) 2011 henrik andersson.
 
 		darktable is free software: you can redistribute it and/or modify
 		it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
 #include <math.h>
 #include <assert.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include "iop/colorout.h"
 #include "develop/develop.h"
 #include "control/control.h"
@@ -31,7 +33,9 @@
 #include "common/opencl.h"
 #include "dtgtk/resetlabel.h"
 
-DT_MODULE(2)
+DT_MODULE(3)
+
+static gchar *_get_profile_from_pos(GList *profiles, int pos);
 
 const char
 *name()
@@ -45,18 +49,51 @@ groups ()
   return IOP_GROUP_COLOR;
 }
 
+static void key_softproof_callback(GtkAccelGroup *accel_group,
+                                   GObject *acceleratable,
+                                   guint keyval, GdkModifierType modifier,
+                                   gpointer data)
+{
+  dt_iop_module_t* self = (dt_iop_module_t*)data;
+  dt_iop_colorout_gui_data_t *g = (dt_iop_colorout_gui_data_t *)self->gui_data;
+  dt_iop_colorout_params_t *p = (dt_iop_colorout_params_t *)self->params;
+
+  /* toggle softproofing on/off */
+  g->softproof_enabled = p->softproof_enabled = !p->softproof_enabled;
+  if(p->softproof_enabled)
+  {
+    int pos = gtk_combo_box_get_active(g->cbox5);
+    gchar *filename = _get_profile_from_pos(g->profiles, pos);
+    if (filename)
+      g_strlcpy(p->softproofprofile, filename, sizeof(p->softproofprofile));
+  }
+
+  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  dt_control_queue_draw_all();
+}
 
 int
 legacy_params (dt_iop_module_t *self, const void *const old_params, const int old_version, void *new_params, const int new_version)
-{
-  if(old_version == 1 && new_version == 2)
+{ 
+  /*  if(old_version == 1 && new_version == 2)
   {
     dt_iop_colorout_params_t *o = (dt_iop_colorout_params_t *)old_params;
     dt_iop_colorout_params_t *n = (dt_iop_colorout_params_t *)new_params;
     memcpy(n,o,sizeof(dt_iop_colorout_params_t));
     n->seq = 0;
     return 0;
+    }*/
+  if(old_version == 2 && new_version == 3)
+  {
+    dt_iop_colorout_params_t *o = (dt_iop_colorout_params_t *)old_params;
+    dt_iop_colorout_params_t *n = (dt_iop_colorout_params_t *)new_params;
+    memcpy(n,o,sizeof(dt_iop_colorout_params_t));
+    n->softproof_enabled = 0;
+    n->softproofintent = 0;
+    g_strlcpy(n->softproofprofile,"sRGB",sizeof(n->softproofprofile));
+    return 0;
   }
+
   return 1;
 }
 
@@ -66,14 +103,14 @@ init_global(dt_iop_module_so_t *module)
   const int program = 2; // basic.cl, from programs.conf
   dt_iop_colorout_global_data_t *gd = (dt_iop_colorout_global_data_t *)malloc(sizeof(dt_iop_colorout_global_data_t));
   module->data = gd;
-  gd->kernel_colorout = dt_opencl_create_kernel(darktable.opencl, program, "colorout");
+  gd->kernel_colorout = dt_opencl_create_kernel(program, "colorout");
 }
 
 void
 cleanup_global(dt_iop_module_so_t *module)
 {
   dt_iop_colorout_global_data_t *gd = (dt_iop_colorout_global_data_t *)module->data;
-  dt_opencl_free_kernel(darktable.opencl, gd->kernel_colorout);
+  dt_opencl_free_kernel(gd->kernel_colorout);
   free(module->data);
   module->data = NULL;
 }
@@ -138,20 +175,15 @@ softproof_profile_changed (GtkComboBox *widget, gpointer user_data)
   dt_iop_colorout_params_t *p = (dt_iop_colorout_params_t *)self->params;
   dt_iop_colorout_gui_data_t *g = (dt_iop_colorout_gui_data_t *)self->gui_data;
   int pos = gtk_combo_box_get_active(widget);
-
   gchar *filename = _get_profile_from_pos(g->profiles, pos);
   if (filename)
   {
-    if (g->softproofprofile)
-      g_free(g->softproofprofile);
-    g->softproofprofile = g_strdup(filename);
-    if(g->softproofing)
-    {
-      p->seq++;
-      dt_dev_add_history_item(darktable.develop, self, TRUE);
-    }
+    g_strlcpy(p->softproofprofile, filename, sizeof(p->softproofprofile));
+    dt_dev_add_history_item(darktable.develop, self, TRUE);
     return;
   }
+
+  fprintf(stderr, "[colorout] softprofile %s seems to have disappeared!\n", p->softproofprofile);
 }
 
 static void
@@ -194,32 +226,36 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
 {
   dt_iop_colorout_data_t *d = (dt_iop_colorout_data_t *)piece->data;
   dt_iop_colorout_global_data_t *gd = (dt_iop_colorout_global_data_t *)self->data;
-  cl_mem dev_m = NULL, dev_r = NULL, dev_g = NULL, dev_b = NULL;
+  cl_mem dev_m = NULL, dev_r = NULL, dev_g = NULL, dev_b = NULL, dev_coeffs = NULL;
 
   cl_int err = -999;
   const int devid = piece->pipe->devid;
   size_t sizes[] = {roi_in->width, roi_in->height, 1};
 
-  dev_m = dt_opencl_copy_host_to_device_constant(sizeof(float)*9, devid, d->cmatrix);
+  dev_m = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*9, d->cmatrix);
   if (dev_m == NULL) goto error;
-  dev_r = dt_opencl_copy_host_to_device(d->lut[0], 256, 256, devid, sizeof(float));
+  dev_r = dt_opencl_copy_host_to_device(devid, d->lut[0], 256, 256, sizeof(float));
   if (dev_r == NULL) goto error;
-  dev_g = dt_opencl_copy_host_to_device(d->lut[1], 256, 256, devid, sizeof(float));
+  dev_g = dt_opencl_copy_host_to_device(devid, d->lut[1], 256, 256, sizeof(float));
   if (dev_g == NULL) goto error;
-  dev_b = dt_opencl_copy_host_to_device(d->lut[2], 256, 256, devid, sizeof(float));
+  dev_b = dt_opencl_copy_host_to_device(devid, d->lut[2], 256, 256, sizeof(float));
   if (dev_g == NULL) goto error;
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, gd->kernel_colorout, 0, sizeof(cl_mem), (void *)&dev_in);
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, gd->kernel_colorout, 1, sizeof(cl_mem), (void *)&dev_out);
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, gd->kernel_colorout, 2, sizeof(cl_mem), (void *)&dev_m);
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, gd->kernel_colorout, 3, sizeof(cl_mem), (void *)&dev_r);
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, gd->kernel_colorout, 4, sizeof(cl_mem), (void *)&dev_g);
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, gd->kernel_colorout, 5, sizeof(cl_mem), (void *)&dev_b);
-  err = dt_opencl_enqueue_kernel_2d(darktable.opencl, devid, gd->kernel_colorout, sizes);
+  dev_coeffs = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*2*3, (float *)d->unbounded_coeffs);
+  if (dev_coeffs == NULL) goto error;
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorout, 0, sizeof(cl_mem), (void *)&dev_in);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorout, 1, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorout, 2, sizeof(cl_mem), (void *)&dev_m);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorout, 3, sizeof(cl_mem), (void *)&dev_r);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorout, 4, sizeof(cl_mem), (void *)&dev_g);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorout, 5, sizeof(cl_mem), (void *)&dev_b);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorout, 6, sizeof(cl_mem), (void *)&dev_coeffs);
+  err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_colorout, sizes);
   if(err != CL_SUCCESS) goto error;
   dt_opencl_release_mem_object(dev_m);
   dt_opencl_release_mem_object(dev_r);
   dt_opencl_release_mem_object(dev_g);
   dt_opencl_release_mem_object(dev_b);
+  dt_opencl_release_mem_object(dev_coeffs);
   return TRUE;
 
 error:
@@ -227,6 +263,7 @@ error:
   if (dev_r != NULL) dt_opencl_release_mem_object(dev_r);
   if (dev_g != NULL) dt_opencl_release_mem_object(dev_g);
   if (dev_b != NULL) dt_opencl_release_mem_object(dev_b);
+  if (dev_coeffs != NULL) dt_opencl_release_mem_object(dev_coeffs);
   dt_print(DT_DEBUG_OPENCL, "[opencl_colorout] couldn't enqueue kernel! %d\n", err);
   return FALSE;
 }
@@ -235,7 +272,7 @@ error:
 void
 process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, void *o, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
-  dt_iop_colorout_data_t *d = (dt_iop_colorout_data_t *)piece->data;
+  const dt_iop_colorout_data_t *const d = (dt_iop_colorout_data_t *)piece->data;
   float *in  = (float *)i;
   float *out = (float *)o;
   const int ch = piece->colors;
@@ -244,7 +281,7 @@ process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, v
   {
     //fprintf(stderr,"Using cmatrix codepath\n");
 #ifdef _OPENMP
-    #pragma omp parallel for schedule(static) default(none) shared(out, roi_out, in, d, i, o)
+    #pragma omp parallel for schedule(static) default(none) shared(out, roi_out, in, i, o)
 #endif
     for (int k=0; k<roi_out->width*roi_out->height; k++)
     {
@@ -260,8 +297,10 @@ process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, v
         rgb[i] = 0.0f;
         for(int j=0; j<3; j++) rgb[i] += d->cmatrix[3*i+j]*XYZ[j];
       }
-      for(int i=0; i<3; i++) out[i] = lerp_lut(d->lut[i], rgb[i]);
-      // for(int i=0;i<3;i++) out[i] = rgb[i];
+      for(int i=0; i<3; i++) out[i] = (d->lut[i][0] >= 0.0f) ?
+        ((rgb[i] < 1.0f) ? lerp_lut(d->lut[i], rgb[i])
+        : dt_iop_eval_exp(d->unbounded_coeffs[i], rgb[i]))
+        : rgb[i];
     }
   }
   else
@@ -344,35 +383,29 @@ static cmsHPROFILE _create_profile(gchar *iccprofile)
 
 void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
-  // pthread_mutex_lock(&darktable.plugin_threadsafe);
   dt_iop_colorout_params_t *p = (dt_iop_colorout_params_t *)p1;
-#ifdef HAVE_GEGL
-  // pull in new params to gegl
-#error "gegl version needs some more care!"
-#else
   dt_iop_colorout_data_t *d = (dt_iop_colorout_data_t *)piece->data;
   gchar *overprofile = dt_conf_get_string("plugins/lighttable/export/iccprofile");
   const int overintent = dt_conf_get_int("plugins/lighttable/export/iccintent");
   const int high_quality_processing = dt_conf_get_bool("plugins/lighttable/export/force_lcms2");
   gchar *outprofile=NULL;
   int outintent = 0;
-  dt_iop_colorout_gui_data_t *g=NULL;
-
-  /* check if we should enable softproofing */
-  if ((g=(dt_iop_colorout_gui_data_t *)self->gui_data) && g->softproofing && g->softproofprofile )
-    d->softproofing = TRUE;
-  else
-    d->softproofing = FALSE;
-
+   
   /* cleanup profiles */
   if (d->output)
     dt_colorspaces_cleanup_profile(d->output);
   d->output = NULL;
 
-  if (d->softproof)
+  if (d->softproof_enabled)
     dt_colorspaces_cleanup_profile(d->softproof);
   d->softproof = NULL;
 
+  d->softproof_enabled = p->softproof_enabled;
+  if(self->dev->gui_attached)
+  {
+    dt_iop_colorout_gui_data_t *g = (dt_iop_colorout_gui_data_t *)self->gui_data;
+    g->softproof_enabled = p->softproof_enabled;
+  }
   const int num_threads = dt_get_num_threads();
   for (int t=0; t<num_threads; t++)
     if (d->xform[t])
@@ -381,6 +414,9 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
       d->xform[t] = NULL;
     }
   d->cmatrix[0] = -0.666f;
+  d->lut[0][0] = -1.0f;
+  d->lut[1][0] = -1.0f;
+  d->lut[2][0] = -1.0f;
   piece->process_cl_ready = 1;
 
   /* if we are exporting then check and set usage of override profile */
@@ -405,8 +441,8 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
   d->output = _create_profile(outprofile);
 
   /* creating softproof profile if softproof is enabled */
-  if (d->softproofing)
-    d->softproof =  _create_profile(g->softproofprofile);
+  if (d->softproof_enabled)
+    d->softproof =  _create_profile(p->softproofprofile);
 
   /*
    * Setup transform flags
@@ -415,12 +451,12 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
 
   /* TODO: the use of bpc should be userconfigurable either from module or preference pane */
   /* softproof flag and black point compensation */
-  transformFlags |= (d->softproofing ? cmsFLAGS_SOFTPROOFING|cmsFLAGS_BLACKPOINTCOMPENSATION : 0);
+  transformFlags |= (d->softproof_enabled ? cmsFLAGS_SOFTPROOFING|cmsFLAGS_BLACKPOINTCOMPENSATION : 0);
       
 
 
   /* get matrix from profile, if softproofing or high quality exporting always go xform codepath */
-  if (d->softproofing || (pipe->type == DT_DEV_PIXELPIPE_EXPORT && high_quality_processing) || 
+  if (d->softproof_enabled || (pipe->type == DT_DEV_PIXELPIPE_EXPORT && high_quality_processing) || 
           dt_colorspaces_get_matrix_from_output_profile (d->output, d->cmatrix, d->lut[0], d->lut[1], d->lut[2], LUT_SAMPLES))
   {
     d->cmatrix[0] = -0.666f;
@@ -440,7 +476,7 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
     if (d->output)
       dt_colorspaces_cleanup_profile(d->output);
     d->output = dt_colorspaces_create_srgb_profile();
-    if (d->softproofing || dt_colorspaces_get_matrix_from_output_profile (d->output, d->cmatrix, d->lut[0], d->lut[1], d->lut[2], LUT_SAMPLES))
+    if (d->softproof_enabled || dt_colorspaces_get_matrix_from_output_profile (d->output, d->cmatrix, d->lut[0], d->lut[1], d->lut[2], LUT_SAMPLES))
     {
       d->cmatrix[0] = -0.666f;
       piece->process_cl_ready = 0;
@@ -454,12 +490,28 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
     }
   }
 
+  // now try to initialize unbounded mode:
+  // we do extrapolation for input values above 1.0f.
+  // unfortunately we can only do this if we got the computation
+  // in our hands, i.e. for the fast builtin-dt-matrix-profile path.
+  for(int k=0;k<3;k++)
+  {
+    // omit luts marked as linear (negative as marker)
+    if(d->lut[k][0] >= 0.0f)
+    {
+      const float x[4] = {0.7f, 0.8f, 0.9f, 1.0f};
+      const float y[4] = {lerp_lut(d->lut[k], x[0]),
+                          lerp_lut(d->lut[k], x[1]),
+                          lerp_lut(d->lut[k], x[2]),
+                          lerp_lut(d->lut[k], x[3])};
+      dt_iop_estimate_exp(x, y, 4, d->unbounded_coeffs[k]);
+    }
+    else d->unbounded_coeffs[k][0] = -1.0f;
+  }
 
   //fprintf(stderr, " Output profile %s, softproof %s%s%s\n", outprofile, d->softproofing?"enabled ":"disabled",d->softproofing?"using profile ":"",d->softproofing?g->softproofprofile:"");
 
-#endif
   g_free(overprofile);
-  // pthread_mutex_unlock(&darktable.plugin_threadsafe);
 }
 
 void init_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -484,7 +536,6 @@ void cleanup_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_de
   // clean up everything again.
   // (void)gegl_node_remove_child(pipe->gegl, piece->input);
 #else
-  // pthread_mutex_lock(&darktable.plugin_threadsafe);
   dt_iop_colorout_data_t *d = (dt_iop_colorout_data_t *)piece->data;
   if(d->output) dt_colorspaces_cleanup_profile(d->output);
   dt_colorspaces_cleanup_profile(d->Lab);
@@ -494,7 +545,6 @@ void cleanup_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_de
 
   free(d->xform);
   free(piece->data);
-  // pthread_mutex_unlock(&darktable.plugin_threadsafe);
 #endif
 }
 
@@ -505,8 +555,8 @@ void gui_update(struct dt_iop_module_t *self)
   dt_iop_colorout_params_t *p = (dt_iop_colorout_params_t *)module->params;
   gtk_combo_box_set_active(g->cbox1, (int)p->intent);
   gtk_combo_box_set_active(g->cbox4, (int)p->displayintent);
-  gtk_combo_box_set_active(g->cbox5, 0);
-  int iccfound = 0, displayfound = 0;
+
+  int iccfound = 0, displayfound = 0, softprooffound = 0;
   GList *prof = g->profiles;
   while(prof)
   {
@@ -521,13 +571,21 @@ void gui_update(struct dt_iop_module_t *self)
       gtk_combo_box_set_active(g->cbox3, pp->pos);
       displayfound = 1;
     }
-    if(iccfound && displayfound) break;
+    if(!strcmp(pp->filename, p->softproofprofile))
+    {
+      gtk_combo_box_set_active(g->cbox5, pp->pos);
+      softprooffound = 1;
+    }
+
+    if(iccfound && displayfound && softprooffound) break;
     prof = g_list_next(prof);
   }
   if(!iccfound)     gtk_combo_box_set_active(g->cbox2, 0);
   if(!displayfound) gtk_combo_box_set_active(g->cbox3, 0);
+  if(!softprooffound) gtk_combo_box_set_active(g->cbox5, 0);
   if(!iccfound)     fprintf(stderr, "[colorout] could not find requested profile `%s'!\n", p->iccprofile);
   if(!displayfound) fprintf(stderr, "[colorout] could not find requested display profile `%s'!\n", p->displayprofile);
+  if(!softprooffound) fprintf(stderr, "[colorout] could not find requested softproof profile `%s'!\n", p->softproofprofile);
 }
 
 void init(dt_iop_module_t *module)
@@ -539,7 +597,8 @@ void init(dt_iop_module_t *module)
   module->priority = 777; // module order created by iop_dependencies.py, do not edit!
   module->hide_enable_button = 1;
   dt_iop_colorout_params_t tmp = (dt_iop_colorout_params_t)
-  {"sRGB", "X profile", DT_INTENT_PERCEPTUAL
+    {"sRGB", "X profile", DT_INTENT_PERCEPTUAL, DT_INTENT_PERCEPTUAL,
+     0, "sRGB",  DT_INTENT_PERCEPTUAL
   };
   memcpy(module->params, &tmp, sizeof(dt_iop_colorout_params_t));
   memcpy(module->default_params, &tmp, sizeof(dt_iop_colorout_params_t));
@@ -556,7 +615,7 @@ void cleanup(dt_iop_module_t *module)
 void gui_post_expose (struct dt_iop_module_t *self, cairo_t *cr, int32_t width, int32_t height, int32_t pointerx, int32_t pointery)
 {
   dt_iop_colorout_gui_data_t *g = (dt_iop_colorout_gui_data_t *)self->gui_data;
-  if(g->softproofing)
+  if(g->softproof_enabled)
   {
     gchar *label=_("SoftProof");
     cairo_set_source_rgba(cr,0.5,0.5,0.5,0.5);
@@ -574,40 +633,8 @@ void gui_post_expose (struct dt_iop_module_t *self, cairo_t *cr, int32_t width,
   }
 }
 
-int  key_pressed(struct dt_iop_module_t *self, uint16_t which)
-{
-  dt_iop_colorout_gui_data_t *g = (dt_iop_colorout_gui_data_t *)self->gui_data;
-  dt_iop_colorout_params_t *p = (dt_iop_colorout_params_t *)self->params;
-  /* toggle softproofing on/off */
-  if (which == KEYCODE_Space)
-  {
-    g->softproofing = !g->softproofing;
-    if(g->softproofing)
-    {
-      int pos = gtk_combo_box_get_active(g->cbox5);
-      gchar *filename = _get_profile_from_pos(g->profiles, pos);
-      if (filename)
-      {
-        if (g->softproofprofile)
-          g_free(g->softproofprofile);
-        g->softproofprofile = g_strdup(filename);
-      }
-    }
-
-
-    /// FIXME: this is certanly the wrong way to do this...
-    p->seq++;
-    dt_dev_add_history_item(darktable.develop, self, TRUE);
-    dt_control_queue_draw_all();
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
 void gui_init(struct dt_iop_module_t *self)
 {
-  // pthread_mutex_lock(&darktable.plugin_threadsafe);
   self->gui_data = malloc(sizeof(dt_iop_colorout_gui_data_t));
   memset(self->gui_data,0,sizeof(dt_iop_colorout_gui_data_t));
   dt_iop_colorout_gui_data_t *g = (dt_iop_colorout_gui_data_t *)self->gui_data;
@@ -657,9 +684,12 @@ void gui_init(struct dt_iop_module_t *self)
       tmpprof = cmsOpenProfileFromFile(filename, "r");
       if(tmpprof)
       {
+	char *lang = getenv("LANG");
+	if (!lang) lang = "en_US";
+
         dt_iop_color_profile_t *prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
         char name[1024];
-        cmsGetProfileInfoASCII(tmpprof, cmsInfoDescription, getenv("LANG"), getenv("LANG")+3, name, 1024);
+        cmsGetProfileInfoASCII(tmpprof, cmsInfoDescription, lang, lang+3, name, 1024);
         g_strlcpy(prof->name, name, sizeof(prof->name));
         g_strlcpy(prof->filename, d_name, sizeof(prof->filename));
         prof->pos = ++pos;
@@ -774,7 +804,15 @@ void gui_init(struct dt_iop_module_t *self)
                     G_CALLBACK (softproof_profile_changed),
                     (gpointer)self);
 
-  // pthread_mutex_unlock(&darktable.plugin_threadsafe);
+
+  // Connecting the accelerator
+  g->softproof_callback = g_cclosure_new(G_CALLBACK(key_softproof_callback),
+                                         (gpointer)self, NULL);
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/plugins/colorout/toggle softproofing",
+      g->softproof_callback);
+
 }
 
 void gui_cleanup(struct dt_iop_module_t *self)
@@ -785,8 +823,22 @@ void gui_cleanup(struct dt_iop_module_t *self)
     g_free(g->profiles->data);
     g->profiles = g_list_delete_link(g->profiles, g->profiles);
   }
+  dt_accel_group_disconnect(darktable.control->accels_darkroom,
+                             ((dt_iop_colorout_gui_data_t*)(self->gui_data))->
+                             softproof_callback);
   free(self->gui_data);
   self->gui_data = NULL;
 }
 
+void init_key_accels()
+{
+  gtk_accel_map_add_entry("<Darktable>/darkroom/plugins/colorout/toggle softproofing",
+                          GDK_s, 0);
+
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/plugins/colorout/toggle softproofing",
+      NULL);
+}
+
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
