@@ -226,23 +226,23 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   cl_int err = -999;
 
 
-  dev_coeffs = dt_opencl_copy_host_to_device_constant(sizeof(float)*3, devid, coeffs);
+  dev_coeffs = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*3, coeffs);
   if (dev_coeffs == NULL) goto error;
 
   size_t sizes[] = {roi_in->width, roi_in->height, 1};
   const int kernel = ui ? gd->kernel_whitebalance_1ui : gd->kernel_whitebalance_4f;
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, kernel, 0, sizeof(cl_mem), (void *)&dev_in);
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, kernel, 1, sizeof(cl_mem), (void *)&dev_out);
-  dt_opencl_set_kernel_arg(darktable.opencl, devid, kernel, 2, sizeof(cl_mem), (void *)&dev_coeffs);
+  dt_opencl_set_kernel_arg(devid, kernel, 0, sizeof(cl_mem), (void *)&dev_in);
+  dt_opencl_set_kernel_arg(devid, kernel, 1, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, kernel, 2, sizeof(cl_mem), (void *)&dev_coeffs);
   if(ui)
   {
-    dt_opencl_set_kernel_arg(darktable.opencl, devid, kernel, 3, sizeof(uint32_t), (void *)&filters);
-    dt_opencl_set_kernel_arg(darktable.opencl, devid, kernel, 4, sizeof(uint32_t), (void *)&roi_out->x);
-    dt_opencl_set_kernel_arg(darktable.opencl, devid, kernel, 5, sizeof(uint32_t), (void *)&roi_out->y);
+    dt_opencl_set_kernel_arg(devid, kernel, 3, sizeof(uint32_t), (void *)&filters);
+    dt_opencl_set_kernel_arg(devid, kernel, 4, sizeof(uint32_t), (void *)&roi_out->x);
+    dt_opencl_set_kernel_arg(devid, kernel, 5, sizeof(uint32_t), (void *)&roi_out->y);
   }
-  err = dt_opencl_enqueue_kernel_2d(darktable.opencl, devid, kernel, sizes);
+  err = dt_opencl_enqueue_kernel_2d(devid, kernel, sizes);
   if(err != CL_SUCCESS) goto error;
-  dt_opencl_finish(darktable.opencl->dev[devid].cmd_queue);
+  dt_opencl_finish(devid);
   dt_opencl_release_mem_object(dev_coeffs);
   for(int k=0; k<3; k++)
     piece->pipe->processed_maximum[k] = d->coeffs[k] * piece->pipe->processed_maximum[k];
@@ -355,15 +355,15 @@ void init_global(dt_iop_module_so_t *module)
   const int program = 2; // basic.cl, from programs.conf
   dt_iop_temperature_global_data_t *gd = (dt_iop_temperature_global_data_t *)malloc(sizeof(dt_iop_temperature_global_data_t));
   module->data = gd;
-  gd->kernel_whitebalance_1ui = dt_opencl_create_kernel(darktable.opencl, program, "whitebalance_1ui");
-  gd->kernel_whitebalance_4f  = dt_opencl_create_kernel(darktable.opencl, program, "whitebalance_4f");
+  gd->kernel_whitebalance_1ui = dt_opencl_create_kernel(program, "whitebalance_1ui");
+  gd->kernel_whitebalance_4f  = dt_opencl_create_kernel(program, "whitebalance_4f");
 }
 
 void init (dt_iop_module_t *module)
 {
   module->params = malloc(sizeof(dt_iop_temperature_params_t));
   module->default_params = malloc(sizeof(dt_iop_temperature_params_t));
-  module->priority = 22; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 44; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_temperature_params_t);
   module->gui_data = NULL;
 }
@@ -379,8 +379,8 @@ void cleanup (dt_iop_module_t *module)
 void cleanup_global(dt_iop_module_so_t *module)
 {
   dt_iop_temperature_global_data_t *gd = (dt_iop_temperature_global_data_t *)module->data;
-  dt_opencl_free_kernel(darktable.opencl, gd->kernel_whitebalance_1ui);
-  dt_opencl_free_kernel(darktable.opencl, gd->kernel_whitebalance_4f);
+  dt_opencl_free_kernel(gd->kernel_whitebalance_1ui);
+  dt_opencl_free_kernel(gd->kernel_whitebalance_4f);
   free(module->data);
   module->data = NULL;
 }
