@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
+    copyright (c) 2009--2011 johannes hanika.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,6 +39,28 @@
 
 DT_MODULE(1)
 
+static void film_strip_key_accel(GtkAccelGroup *accel_group,
+                                 GObject *acceleratable,
+                                 guint keyval, GdkModifierType modifier,
+                                 gpointer data);
+
+static void zoom_key_accel(GtkAccelGroup *accel_group, GObject *acceleratable,
+                           guint keyval, GdkModifierType modifier,
+                           gpointer data);
+
+static void export_key_accel_callback(GtkAccelGroup *accel_group,
+                                      GObject *acceleratable, guint keyval,
+                                      GdkModifierType modifier,gpointer data);
+
+static void skip_f_key_accel_callback(GtkAccelGroup *accel_group,
+                                      GObject *acceleratable,
+                                      guint keyval, GdkModifierType modifier,
+                                      gpointer data);
+static void skip_b_key_accel_callback(GtkAccelGroup *accel_group,
+                                      GObject *acceleratable,
+                                      guint keyval, GdkModifierType modifier,
+                                      gpointer data);
+
 const char
 *name(dt_view_t *self)
 {
@@ -52,6 +74,59 @@ init(dt_view_t *self)
   self->data = malloc(sizeof(dt_develop_t));
   dt_dev_init((dt_develop_t *)self->data, 1);
 
+  // Film strip shortcuts
+  gtk_accel_map_add_entry("<Darktable>/darkroom/toggle film strip",
+                          GDK_f, GDK_CONTROL_MASK);
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/toggle film strip",
+      NULL);
+
+  // Zoom shortcuts
+  gtk_accel_map_add_entry("<Darktable>/darkroom/zoom/close",
+                          GDK_1, GDK_MOD1_MASK);
+  gtk_accel_map_add_entry("<Darktable>/darkroom/zoom/fill",
+                          GDK_2, GDK_MOD1_MASK);
+  gtk_accel_map_add_entry("<Darktable>/darkroom/zoom/fit",
+                          GDK_3, GDK_MOD1_MASK);
+
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/zoom/close",
+      NULL);
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/zoom/fill",
+      NULL);
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/zoom/fit",
+      NULL);
+
+  // enable shortcut to export with current export settings:
+  gtk_accel_map_add_entry("<Darktable>/darkroom/export",
+                          GDK_e, GDK_CONTROL_MASK);
+
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/export",
+      NULL);
+
+
+  // Shortcut to skip images
+  gtk_accel_map_add_entry("<Darktable>/darkroom/image forward",
+                          GDK_space, 0);
+  gtk_accel_map_add_entry("<Darktable>/darkroom/image back",
+                          GDK_BackSpace, 0);
+
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/image forward",
+      NULL);
+  dt_accel_group_connect_by_path(
+      darktable.control->accels_darkroom,
+      "<Darktable>/darkroom/image back",
+      NULL);
 }
 
 
@@ -377,14 +452,6 @@ select_this_image(const int imgid)
   }
 }
 
-/**
- * \brief Switch to the specified image
- *
- * Switches to the specified image, saving the state of the current image if needed.
- *
- * \param dev The developer
- * \param image The image to switch to
- */
 static void
 dt_dev_change_image(dt_develop_t *dev, dt_image_t *image)
 {
@@ -397,28 +464,20 @@ dt_dev_change_image(dt_develop_t *dev, dt_image_t *image)
     dt_conf_set_string("plugins/darkroom/active", "");
   g_assert(dev->gui_attached);
 
-  // only save image/settings if image was modified
-  if (dev->image && dev->image->dirty)
+  // commit image ops to db
+  dt_dev_write_history(dev);
+
+  // be sure light table will update the thumbnail
+  if(dev->image)
   {
-    // tag image as changed
-    // only tag the image when there was a real change.
-    guint tagid = 0;
-    dt_tag_new("darktable|changed", &tagid);
-    dt_tag_attach(tagid, dev->image->id);
-    // commit image ops to db
-    dt_dev_write_history(dev);
-    // write .xmp file
-    dt_image_write_sidecar_file(dev->image->id);
-
-    // commit updated mipmaps to db
-    // TODO: bg process?
-    dt_dev_process_to_mip(dev);
+    dt_dev_get_processed_size(dev, &dev->image->output_width, &dev->image->output_height);
+    dev->image->force_reimport = 1;
   }
-
   // release full buffer
   if(dev->image && dev->image->pixels)
     dt_image_release(dev->image, DT_IMAGE_FULL, 'r');
 
+  // writes the .xmp and the database:
   dt_image_cache_flush(dev->image);
 
   dev->image = image;
@@ -531,12 +590,6 @@ film_strip_activated(const int imgid, void *data)
   dt_view_film_strip_prefetch();
 }
 
-/**
- * \brief Jump forward (diff) images in the collection
- *
- * \param[in] dev A pointer to the dt_develop_t to use for state
- * \param[in] diff The number of images to jump forward.  Use negative values to jump backward.
- */
 static void
 dt_dev_jump_image(dt_develop_t *dev, int diff)
 {
@@ -594,7 +647,9 @@ dt_dev_jump_image(dt_develop_t *dev, int diff)
 }
 
 static void
-zoom_key_accel(void *data)
+zoom_key_accel(GtkAccelGroup *accel_group,
+               GObject *acceleratable, guint keyval,
+               GdkModifierType modifier, gpointer data)
 {
   dt_develop_t *dev = darktable.develop;
   int zoom, closeup;
@@ -630,20 +685,129 @@ zoom_key_accel(void *data)
 }
 
 static void
-film_strip_key_accel(void *data)
+film_strip_key_accel(GtkAccelGroup *accel_group,
+                     GObject *acceleratable, guint keyval,
+                     GdkModifierType modifier, gpointer data)
 {
-  dt_view_film_strip_toggle(darktable.view_manager, film_strip_activated, data);
+  dt_view_film_strip_toggle(darktable.view_manager, film_strip_activated,
+                            (void*)data);
   dt_control_queue_draw_all();
 }
 
 static void
-export_key_accel_callback(void *d)
+export_key_accel_callback(GtkAccelGroup *accel_group,
+                          GObject *acceleratable, guint keyval,
+                          GdkModifierType modifier, gpointer data)
 {
+  /* write history before exporting */
+  dt_dev_write_history((dt_develop_t *)data);
+
+  /* export current image */
   dt_control_export();
 }
 
+static void skip_f_key_accel_callback(GtkAccelGroup *accel_group,
+                                      GObject *acceleratable, guint keyval,
+                                      GdkModifierType modifier, gpointer data)
+{
+  dt_dev_jump_image((dt_develop_t*)data, 1);
+}
+
+static void skip_b_key_accel_callback(GtkAccelGroup *accel_group,
+                                      GObject *acceleratable,
+                                      guint keyval, GdkModifierType modifier,
+                                      gpointer data)
+{
+  dt_dev_jump_image((dt_develop_t*)data, -1);
+}
+
+
+static void show_module_callback(GtkAccelGroup *accel_group,
+                                 GObject *acceleratable,
+                                 guint keyval, GdkModifierType modifier,
+                                 gpointer data)
+
+{
+  dt_iop_module_t *module = (dt_iop_module_t*)data;
+
+  // Showing the module, if it isn't already visible
+  if(!dtgtk_tristatebutton_get_state(DTGTK_TRISTATEBUTTON(module->showhide)))
+  {
+    dtgtk_tristatebutton_set_state(DTGTK_TRISTATEBUTTON(module->showhide), 1);
+    gtk_widget_queue_draw(module->showhide);
+  }
+
+  dt_gui_iop_modulegroups_switch(module->groups());
+  gtk_expander_set_expanded(GTK_EXPANDER(module->expander), TRUE);
+  dt_iop_request_focus(module);
+}
+
+static void connect_closures(dt_view_t *self)
+{
+  GClosure *closure;
+  dt_develop_t *dev = (dt_develop_t*)self->data;
+
+  // Film strip shortcuts
+  closure = g_cclosure_new(G_CALLBACK(film_strip_key_accel),
+                           (gpointer)self, NULL);
+  dev->closures = g_slist_prepend(dev->closures, closure);
+  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                 "<Darktable>/darkroom/toggle film strip",
+                                 closure);
+
+  // Zoom shortcuts
+  closure = g_cclosure_new(G_CALLBACK(zoom_key_accel), (gpointer)1, NULL);
+  dev->closures = g_slist_prepend(dev->closures, closure);
+  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                 "<Darktable>/darkroom/zoom/close",
+                                 closure);
+  closure = g_cclosure_new(G_CALLBACK(zoom_key_accel), (gpointer)2, NULL);
+  dev->closures = g_slist_prepend(dev->closures, closure);
+  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                 "<Darktable>/darkroom/zoom/fill",
+                                 closure);
+  closure = g_cclosure_new(G_CALLBACK(zoom_key_accel), (gpointer)3, NULL);
+  dev->closures = g_slist_prepend(dev->closures, closure);
+  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                 "<Darktable>/darkroom/zoom/fit",
+                                 closure);
+
+  // enable shortcut to export with current export settings:
+  closure = g_cclosure_new(G_CALLBACK(export_key_accel_callback), 
+			   (gpointer)self->data, NULL);
+  dev->closures = g_slist_prepend(dev->closures, closure);
+  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                 "<Darktable>/darkroom/export",
+                                 closure);
+
+
+  // Shortcut to skip images
+  closure = g_cclosure_new(G_CALLBACK(skip_f_key_accel_callback),
+                           (gpointer)self->data, NULL);
+  dev->closures = g_slist_prepend(dev->closures, closure);
+  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                 "<Darktable>/darkroom/image forward",
+                                 closure);
+  closure = g_cclosure_new(G_CALLBACK(skip_b_key_accel_callback),
+                           (gpointer)self->data, NULL);
+  dev->closures = g_slist_prepend(dev->closures, closure);
+  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                 "<Darktable>/darkroom/image back",
+                                 closure);
+
+}
+
 void enter(dt_view_t *self)
 {
+  char accelpath[256];
+
+  // Attaching accelerator group
+  gtk_window_add_accel_group(GTK_WINDOW(darktable.gui->widgets.main_window),
+                             darktable.control->accels_darkroom);
+
+  // Connecting the closures
+  connect_closures(self);
+
   dt_print(DT_DEBUG_CONTROL, "[run_job+] 11 %f in darkroom mode\n", dt_get_wtime());
   dt_develop_t *dev = (dt_develop_t *)self->data;
 
@@ -697,8 +861,16 @@ void enter(dt_view_t *self)
     GtkWidget *expander = dt_iop_gui_get_expander(module);
     module->topwidget = GTK_WIDGET(expander);
     gtk_box_pack_start(box, expander, FALSE, FALSE, 0);
+    module->show_closure = NULL;
     if(strcmp(module->op, "gamma") && !(module->flags() & IOP_FLAGS_DEPRECATED))
     {
+      // Connecting the (optional) module show accelerator
+      snprintf(accelpath, 256, "<Darktable>/darkroom/plugins/%s/show plugin", module->op);
+      module->show_closure = g_cclosure_new(G_CALLBACK(show_module_callback),
+                                                       module, NULL);
+      dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                     accelpath, module->show_closure);
+
       module->showhide = dtgtk_tristatebutton_new(NULL,0);
       char filename[1024], datadir[1024];
       dt_get_datadir(datadir, 1024);
@@ -783,13 +955,6 @@ void enter(dt_view_t *self)
     dt_view_film_strip_open(darktable.view_manager, film_strip_activated, self);
     dt_view_film_strip_prefetch();
   }
-  dt_gui_key_accel_register(GDK_CONTROL_MASK, GDK_f, film_strip_key_accel, self);
-  dt_gui_key_accel_register(GDK_MOD1_MASK, GDK_1, zoom_key_accel, (void *)1);
-  dt_gui_key_accel_register(GDK_MOD1_MASK, GDK_2, zoom_key_accel, (void *)2);
-  dt_gui_key_accel_register(GDK_MOD1_MASK, GDK_3, zoom_key_accel, (void *)3);
-
-  // enable shortcut to export with current export settings:
-  dt_gui_key_accel_register(GDK_CONTROL_MASK, GDK_e, export_key_accel_callback, NULL);
 
   // switch on groups as they where last time:
   dt_gui_iop_modulegroups_switch(dt_conf_get_int("plugins/darkroom/groups"));
@@ -825,6 +990,21 @@ dt_dev_remove_child(GtkWidget *widget, gpointer data)
 
 void leave(dt_view_t *self)
 {
+  GSList *c = ((dt_develop_t*)self->data)->closures;
+
+  // Detaching the accelerator group
+  gtk_window_remove_accel_group(GTK_WINDOW(darktable.gui->widgets.main_window),
+                                darktable.control->accels_darkroom);
+
+  // Disconnecting and deleting the closures
+  while(c)
+  {
+    dt_accel_group_disconnect(darktable.control->accels_darkroom, c->data);
+    c = g_slist_next(c);
+  }
+  g_slist_free(((dt_develop_t*)self->data)->closures);
+  ((dt_develop_t*)self->data)->closures = NULL;
+
   // store groups for next time:
   dt_conf_set_int("plugins/darkroom/groups", dt_gui_iop_modulegroups_get());
 
@@ -836,9 +1016,6 @@ void leave(dt_view_t *self)
 
   if(dt_conf_get_bool("plugins/filmstrip/on"))
     dt_view_film_strip_close(darktable.view_manager);
-  dt_gui_key_accel_unregister(film_strip_key_accel);
-  dt_gui_key_accel_unregister(zoom_key_accel);
-  dt_gui_key_accel_unregister(export_key_accel_callback);
 
   GList *childs = gtk_container_get_children (
                     GTK_CONTAINER (darktable.gui->widgets.bottom_left_toolbox));
@@ -874,11 +1051,13 @@ void leave(dt_view_t *self)
   dt_tag_attach(tagid, dev->image->id);
   // commit image ops to db
   dt_dev_write_history(dev);
-  // write .xmp file
-  dt_image_write_sidecar_file(dev->image->id);
 
-  // commit updated mipmaps to db
-  dt_dev_process_to_mip(dev);
+  // be sure light table will regenerate the thumbnail:
+  if(dev->image)
+  {
+    dt_dev_get_processed_size(dev, &dev->image->output_width, &dev->image->output_height);
+    dev->image->force_reimport = 1;
+  }
 
   // clear gui.
   dev->gui_leaving = 1;
@@ -903,6 +1082,11 @@ void leave(dt_view_t *self)
     snprintf(var, 1024, "plugins/darkroom/%s/expanded", module->op);
     dt_conf_set_bool(var, gtk_expander_get_expanded (module->expander));
 
+    // disconnect the show accelerator
+    if(module->show_closure)
+      dt_accel_group_disconnect(darktable.control->accels_darkroom,
+                                module->show_closure);
+
     module->gui_cleanup(module);
     dt_iop_cleanup_module(module) ;
     free(module);
@@ -1006,6 +1190,11 @@ void mouse_moved(dt_view_t *self, double x, double y, int which)
 int button_released(dt_view_t *self, double x, double y, int which, uint32_t state)
 {
   dt_develop_t *dev = darktable.develop;
+  const int32_t width_i  = self->width;
+  const int32_t height_i = self->height;
+  if(width_i  > DT_IMAGE_WINDOW_SIZE) x += (DT_IMAGE_WINDOW_SIZE-width_i) *.5f;
+  if(height_i > DT_IMAGE_WINDOW_SIZE) y += (DT_IMAGE_WINDOW_SIZE-height_i)*.5f;
+
   int handled = 0;
   if(dev->gui_module && dev->gui_module->button_released) handled = dev->gui_module->button_released(dev->gui_module, x, y, which, state);
   if(handled) return handled;
@@ -1083,6 +1272,11 @@ int button_pressed(dt_view_t *self, double x, double y, int which, int type, uin
 void scrolled(dt_view_t *self, double x, double y, int up, int state)
 {
   dt_develop_t *dev = (dt_develop_t *)self->data;
+  const int32_t width_i  = self->width;
+  const int32_t height_i = self->height;
+  if(width_i  > DT_IMAGE_WINDOW_SIZE) x += (DT_IMAGE_WINDOW_SIZE-width_i) *.5f;
+  if(height_i > DT_IMAGE_WINDOW_SIZE) y += (DT_IMAGE_WINDOW_SIZE-height_i)*.5f;
+
   int handled = 0;
   if(dev->gui_module && dev->gui_module->scrolled) handled = dev->gui_module->scrolled(dev->gui_module, x, y, up, state);
   if(handled) return;
@@ -1161,22 +1355,9 @@ void border_scrolled(dt_view_t *view, double x, double y, int which, int up)
 }
 
 
-int key_pressed(dt_view_t *self, uint16_t which)
+int key_pressed(dt_view_t *self, guint key, guint state)
 {
-  dt_develop_t *dev = (dt_develop_t *)self->data;
-  int handled = 0;
-  if(dev->gui_module && dev->gui_module->key_pressed) handled = dev->gui_module->key_pressed(dev->gui_module, which);
-  if(handled) return handled;
-  switch(which)
-  {
-    case KEYCODE_Space:
-      dt_dev_jump_image(dev, 1);
-      return 1;
-    case KEYCODE_BackSpace:
-      dt_dev_jump_image(dev, -1);
-      return 1;
-  }
-  return 0;
+  return 1;
 }
 
 
